#!/usr/bin/python
# -*- coding: utf-8 -*-

# Author:
# Linwood Creekmore
# Email: valinvescap@gmail.com

##############################
# Standard Library Import
##############################

import platform
try:
    from unittest import TestCase, mock
except:
    from unittest import TestCase
    import mock

import os
from unittest import TestCase

import pandas as pd

import gdelt
from gdelt.vectorizingFuncs import _urlBuilder, _geofilter

##############################
# Third Party Libraries
##############################

try:
    import geopandas
except:
    print('geopandas not installed')

##############################
# Custom Library Import
##############################


#################################
# Test files
#################################

# can't read compressed pickles in old pandas version
try:
    test_df = pd.read_pickle(os.path.join(
        gdelt.base.BASE_DIR, "data", "events2samp.gz"),
        compression="gzip").drop('CAMEOCodeDescription', axis=1)

except:

    if platform.python_version_tuple()[1] == '4':
        import gzip
        import tempfile

        fh = gzip.GzipFile(os.path.join(
            gdelt.base.BASE_DIR, "data", "events2samp.gz"))
        with tempfile.NamedTemporaryFile() as tmp:
            lines = fh.read()
            tmp.write(lines)
            test_df = pd.read_pickle(tmp.name)

    else:
        test_df = pd.read_pickle(os.path.join(
            gdelt.base.BASE_DIR, "data", "events2samp.gz"),
            compression="gzip").drop('CAMEOCodeDescription', axis=1)


class testGDELTVectorizingFuncs(TestCase):

    # print(test_df)

    def test_gdeltdate_urlbuilder_events_v1_three(self):
        date_sequence = ['20160101234500', '20160102234500', '20160103234500']
        exp = [
            'http://data.gdeltproject.org/events/20160101234500.export.CSV.zip',
            'http://data.gdeltproject.org/events/20160102234500.export.CSV.zip',
            'http://data.gdeltproject.org/events/20160103234500.export.CSV.zip']

        result = _urlBuilder(['20160101234500', '20160102234500',
                              '20160103234500'],version=1)

        return self.assertEqual(exp, result, "Events 1 more than 2 failed.")

    def test_gdeltdate_urlbuilder_events_v1_fail_single(self):
        date_sequence = '201301'

        exp = ("GDELT 1.0 Global Knowledge Graph requires dates greater than "
               "or equal to April 1 2013")

        with self.assertRaises(Exception) as context:
            result = _urlBuilder(date_sequence, version=1, table='gkg')
        the_exception = context.exception
        return self.assertEqual(exp, str(the_exception),
                                "Version 1 old, gkg fail.")

    def test_gdeltdate_urlbuilder_events_v1_fail_list(self):
        date_sequence = ['201301', '201302']

        exp = ("GDELT 1.0 Global Knowledge Graph requires dates greater than "
               "or equal to April 1 2013")

        with self.assertRaises(Exception) as context:
            result = _urlBuilder(date_sequence, version=1, table='gkg')
        the_exception = context.exception

        return self.assertEqual(exp, str(the_exception),
                                "Version 1 old, gkg fail.")

    # def test_gdelt_geofilter_events2(self):
    #     """Test ability to turn df into geodataframe"""
    #
    #     # load df
    #     spam = pd.read_pickle(os.path.join(
    #         gdelt.base.BASE_DIR, "data", "events2samp.gz"),
    #         compression="gzip").drop('CAMEOCodeDescription', axis=1)
    #     # print(spam)
    #
    #     try:
    #         import geopandas as gpd
    #         geo = _geofilter(spam)
    #
    #         return self.assertIsInstance(geo,gpd.GeoDataFrame)
    #     except:
    #         warnings.warn('Geopandas is not installed. '
    #                       'Unable to test _geofilter')
    #
    #         exp = "Blank"
    #         return self.assertEqual("Blank",exp)
    def test_geopandas_import_pass(self):
        """Test ability to import geopandas and create spatial object"""
        #import geopandas as gpd

        # exp = "geopandas is not installed. gdeltPyR needs geopandas to export as shapefile. Visit http://geopandas.org/install.html for instructions."
        import geopandas as gpd
        gdf = _geofilter(test_df)

        return self.assertTrue(isinstance(gdf, gpd.geodataframe.GeoDataFrame))

    def test_geopandas_import_pass_normcol(self):
        import geopandas as gpd
        test_df.columns = list(
            map(lambda x: (x.replace('_', "")).lower(), test_df.columns))
        gdf = _geofilter(test_df)
        return self.assertTrue(isinstance(gdf, gpd.geodataframe.GeoDataFrame))


class testFailGeopandas(TestCase):

    def test_geopandas_import_fail(self):
        """Test ability to import geopandas"""
        import sys
        sys.modules['geopandas'] = None
        exp = "geopandas is not installed. gdeltPyR needs geopandas to export as shapefile. Visit http://geopandas.org/install.html for instructions."
        with self.assertRaises(Exception) as context:
            gdf = _geofilter(test_df)
        the_exception = context.exception

        return self.assertTrue(exp, str(the_exception))



#!/usr/bin/python

# Author:
# Linwood Creekmore
# Email: valinvescap@gmail.com


##################################
# Standard library imports
##################################
import datetime
import sys
import traceback

##################################
# Third party imports
##################################
import numpy as np
from dateutil.parser import parse


def _date_input_check(date, version):
    """Function to check date entered by user.

    Parameters
    ----------
        date : {string or list},
            Input data, where ``date`` is a single date string,
            two dates representing a range, or several dates
            that represent individual days of interest.

        version: {int},
            An integer between 1 and 2 representing the version
            of GDELT used.
    Returns
    -------
    self : None
        Returns self.
        :param date:
        :param version:
    """

    if isinstance(date, str):

        if date != "":
            if parse(date) > datetime.datetime.now():  # pragma: no cover
                raise ValueError(
                    'Your date is greater than the current date. '
                    'Please enter a relevant date.'
                )
            elif parse(date) < parse('Feb 18 2015') and int(
                    version) != 1:  # pragma: no cover
                raise ValueError(
                    "GDELT 2.0 only supports \'Feb 18 2015 - Present\' "
                    "queries currently. Try another date."
                )
        if version == 1 and parse(date).date() == \
                datetime.datetime.now().date():
            raise ValueError(
                ("You entered today's date for a GDELT 1.0 query. GDELT 1.0's "
                 "most recent data is always the"
                 " trailing day (i.e. {0}).  Please retry your query.").format(
                    np.datetime64(datetime.datetime.now().date()) -
                    np.timedelta64(1, 'D'))
            )
        # GDELT release yesterday's data 6AM today
        if datetime.datetime.now().hour <= 6 and parse(date).date() == (
                    datetime.datetime.now().date() - datetime.timedelta(
            days=1)) and version == 1:  # pragma: no cover
            raise BaseException('GDELT 1.0 posts the latest daily update by '
                                '6AM EST.'
                                'The next update will appear in {0}'.format(
                str(datetime.datetime.combine(
                    datetime.datetime.now(), datetime.datetime.min.time()
                ) + datetime.timedelta(hours=6, minutes=00, seconds=00) -
                    datetime.datetime.now())))

    elif isinstance(date, list) or isinstance(date, np.ndarray):
        newdate =[]

        for l in date:
            if len(l) == 4:
                test = (str(datetime.datetime.strptime(l, '%Y')))
                newdate.append(test)
            elif len(l) == 6:  # pragma: no cover
                test = str(datetime.datetime.strptime(l, '%Y%m'))
                newdate.append(test)
            else:
                try:
                    test = str(parse(str(l)))
                except:  # pragma: no cover
                    test = l
                newdate.append(test)
            if parse(test) < parse('Feb 18 2015') and version == 2:
                raise ValueError(
                    "GDELT 2.0 only supports \'Feb 18 2015 - "
                    "Present\'queries currently. Try another date."
                )
        date = newdate

        if len(date) == 1:
            try:
                if parse("".join(date)) > datetime.datetime.now():
                    raise ValueError(
                        'Your date is greater than the current date.  '
                        'Please enter a relevant date.'
                    )
                elif version == 2 and parse("".join(date)) < \
                        parse('Feb 18 2015') and int(
                    version) != 1:  # pragma: no cover
                    raise ValueError(
                        "GDELT 2.0 only supports \'Feb 18 2015 - "
                        "Present\'queries currently. Try another date."
                    )

            except:
                # exc_type, exc_value, exc_traceback = sys.exc_info()
                # traceback.print_tb(exc_traceback, limit=1, file=sys.stdout)
                # traceback.print_exception(exc_type, exc_value, exc_traceback,
                #                           limit=2, file=sys.stdout)

                raise ValueError(
                    "One or more of your input date strings does not parse to "
                    "a date format. Check input."
                )
            # if datetime.datetime.now().hour <= 6 and parse(
            #         "".join(date)).date() == (
            #             datetime.datetime.now().date() - datetime.timedelta(
            #             days=1)):
            #     raise BaseException('GDELT 1.0 posts the latest daily update '
            #                         'by 6AM EST. The next update will appear '
            #                         'in {0}'.format(str(
            #         datetime.datetime.combine(
            #             datetime.datetime.now(), datetime.datetime.min.time()
            #         ) + datetime.timedelta(hours=6, minutes=00, seconds=00) -
            #         datetime.datetime.now())))
            #
            # return "".join(date)


        elif len(date) == \
                2 and (isinstance(date, list) or isinstance(date, np.ndarray)):

            try:
                list(map(parse, date))
            except Exception as exc:  # pragma: no cover
                exc_type, exc_value, exc_traceback = sys.exc_info()
                traceback.print_tb(exc_traceback, limit=1, file=sys.stdout)
                traceback.print_exception(exc_type, exc_value, exc_traceback,
                                          limit=2, file=sys.stdout)
                raise ValueError(
                    "One or more of your input date strings does not parse to "
                    "a date format. Check input."
                )

            if not bool(parse(date[0]) < parse(date[1])):
                raise ValueError(
                    'Start date greater than or equal to end date. '
                    'Check your entered date query.'
                )

            elif not np.all(
                    np.logical_not(np.array(list(map(parse, date))) >
                                           datetime.datetime.now())
            ):
                raise ValueError(
                    "One of your dates is greater than the current date. "
                    "Check your entered date query."
                )
            # elif np.any(
            #         np.logical_not(np.array(list(map(parse, date)
            #                                      )) > parse("Feb 18 2015"))) == \
            #         True and int(version) != 1:
            #     raise ValueError(
            #         "GDELT 2.0 only supports \'Feb 18 2015 - Present\'queries "
            #         "currently. Try another date."
            #     )

            # elif version == 1:
            # #     if not np.all(
            # #             np.logical_not(np.array(list(map(lambda x: parse(x),
            # #                                              date)),
            # #                                     dtype='datetime64[D]') >=
            # #                                    np.datetime64(
            # #                                        datetime.datetime.now().date()))):
            # #         raise ValueError(
            # #             "You have today's date in your query for GDELT 1.0. "
            # #             " GDELT 1.0\'s most recent data"
            # #             "is always the trailing day (i.e. {0}).  Please retry "
            # #             "your query.".format(
            # #                 np.datetime64(datetime.datetime.now().date()) -
            # #                 np.timedelta64(1, 'D'))
            # #         )
            #     if datetime.datetime.now().hour <= 6 and (
            #                 datetime.datetime.now().date() - datetime.timedelta(
            #                 days=1)) in list(
            #         map(lambda x: parse(x).date(), date)):
            #         if datetime.datetime.now().hour < 6:
            #             raise BaseException('GDELT 1.0 posts the latest daily '
            #                                 'update by 6AM EST.  The next '
            #                                 'update will appear in {0}'.format(
            #                 str(datetime.datetime.combine(
            #                     datetime.datetime.now(),
            #                     datetime.datetime.min.time()
            #                 ) + datetime.timedelta(
            #                     hours=6, minutes=00, seconds=00) -
            #                     datetime.datetime.now())))


        elif len(date) > 2:

            try:
                map(parse, date)
            except Exception as exc:  # pragma: no cover
                exc_type, exc_value, exc_traceback = sys.exc_info()
                traceback.print_tb(exc_traceback, limit=1, file=sys.stdout)
                traceback.print_exception(exc_type, exc_value, exc_traceback,
                                          limit=2, file=sys.stdout)
                raise ValueError(
                    "One or more of your input date strings does not parse "
                    "to a date format. Check input."
                )

            if not np.all(
                    np.logical_not(np.array(list(map(parse, date))) >
                                           datetime.datetime.now())
            ):
                raise ValueError(
                    "One or more of your input date strings is greater than"
                    " today's date. Check input."
                )

            elif np.any(
                    np.logical_not(np.array(list(map(parse, date)
                                                 )) > parse("Feb 18 2015"))) == \
                    True and int(version) != 1:  # pragma: no cover
                raise ValueError(
                    "GDELT 2.0 only supports \'Feb 18 2015 - Present\'queries "
                    "currently. Try another date."
                )
            elif version == 1:

                if not np.all(
                        np.logical_not(np.array(list(map(lambda x: parse(x),
                                                         date)),
                                                dtype='datetime64[D]') >=
                                               np.datetime64(
                                                   datetime.datetime.now().date()))):  # pragma: no cover
                    raise ValueError(
                        "You have today's date in your query for GDELT 1.0. "
                        "GDELT 1.0\'s most recent data"
                        "is always the trailing day (i.e. {0}).  Please "
                        "retry your query.".format(
                            np.datetime64(
                                datetime.datetime.now().date()) -
                            np.timedelta64(1, 'D'))
                    )
                if datetime.datetime.now().hour <= 6 and (
                            datetime.datetime.now().date() - datetime.timedelta(
                            days=1)) in list(map(
                    lambda x: parse(x).date(), date)):  # pragma: no cover
                    if datetime.datetime.now().hour < 6:
                        raise BaseException('GDELT 1.0 posts the latest daily '
                                            'update by 6AM EST.'
                                            'The next update will appear in '
                                            '{0}'.format(str(
                            datetime.datetime.combine(
                                datetime.datetime.now(),
                                datetime.datetime.min.time()
                            ) + datetime.timedelta(
                                hours=6, minutes=00, seconds=00) -
                            datetime.datetime.now())))



# #!/usr/bin/python
# # -*- coding: utf-8 -*-
#
# Author:
# Linwood Creekmore
# Email: valinvescap@gmail.com

##############################
# Standard Library Import
##############################

import datetime
import os
import platform
from unittest import TestCase

import numpy as np
import pandas as pd

import gdelt
from gdelt.helpers import _rooturl, _shaper, _cameos, _testdate, _tableinfo

##############################
# Third Party Libraries
##############################
##############################
# Custom Library Import
##############################

##############################
# Directories
##############################

this_dir, this_filename = os.path.split(__file__)
BASE_DIR = os.path.dirname(this_dir)


class testHelpers(TestCase):
    """Test helper functions"""
    def test_date(self):

        res = _testdate('1980')
        return self.assertIsInstance(res,datetime.datetime)


    def test_urlgetter(self):
        """Pull the root url from a news story url"""
        dd = pd.read_pickle(os.path.join(
            gdelt.base.BASE_DIR, "data", "events2samp.gz"),
            compression="gzip").drop('CAMEOCodeDescription', axis=1)
        test = dd.drop_duplicates('SOURCEURL')[:9]

        exp = np.array(
            ['www.cvilletomorrow.org', 'www.thetowntalk.com',
             'www.comicsbeat.com', 'www.nbcnews.com', 'www.thearabweekly.com',
             'www.thestar.com.my', 'www.thetimes.co.uk',
             'timesofindia.indiatimes.com', 'www.woad.com'], dtype=object)

        resp = test.apply(_rooturl, axis=1).values
        return np.testing.assert_array_equal(exp, resp)

    def test_shaper(self):
        """Make the points"""
        if platform.python_version_tuple()[1] == '4':
            return self.assertIsNone(None)
        dd = pd.read_pickle(os.path.join(
            gdelt.base.BASE_DIR, "data", "events2samp.gz"),
            compression="gzip").drop('CAMEOCodeDescription', axis=1)

        try:
            # Applying the function
            fin = dd.apply(_shaper, axis=1)

            return self.assertIsInstance(fin, pd.Series)
        except:
            exp= ('You need to install shapely to use this feature.')
            with self.assertRaises(Exception) as context:
                fin = dd.apply(_shaper, axis=1)
            the_exception = context.exception


            return self.assertIsInstance(str(the_exception),str, "Not installed")


    def test_cameo(self):
        """Make the cameo code description"""
        codes = pd.read_json(os.path.join(
            gdelt.base.BASE_DIR, "data", "cameoCodes.json"))

        dd = pd.read_pickle(os.path.join(
            gdelt.base.BASE_DIR, "data", "events2samp.gz"),
            compression="gzip").drop('CAMEOCodeDescription', axis=1)
        da = dd.apply(lambda x: _cameos(dd,codes))
        return self.assertIsInstance(da,pd.Series)

    def testtable_events2(self):
        """Make the cameo code description"""
        tabs = pd.read_csv(
            os.path.join(BASE_DIR, 'data', 'events2.csv'))
        return self.assertTrue(tabs.equals(_tableinfo('events',version=2)))

    def testtable_gkg2(self):
        """Make the cameo code description"""
        tabs = pd.read_csv(
            os.path.join(BASE_DIR, 'data', 'gkg2.csv'))
        return self.assertTrue(tabs.equals(_tableinfo('gkg',version=2)))

    def testtable_events1(self):
        """Make the cameo code description"""
        tabs = pd.read_csv(
            os.path.join(BASE_DIR, 'data', 'events1.csv'))
        return self.assertTrue(tabs.equals(_tableinfo('events',version=1)))

    def testtable_mentspass(self):
        """Make the cameo code description"""
        tabs = pd.read_csv(
            os.path.join(BASE_DIR, 'data', 'mentions.csv'))
        return self.assertTrue(tabs.equals(_tableinfo('ments',version=2)))

    def testtable_mentsfail(self):
        """Make the cameo code description"""
        exp = 'GDELT 1.0 does not have a mentions table.'
        with self.assertRaises(Exception) as context:
            checked = _tableinfo('ments',version=1)
        the_exception = context.exception

        return self.assertEqual(exp, str(the_exception))

    def testtable_cloudviz(self):
        """Make the cameo code description"""
        tabs = pd.read_csv(
            os.path.join(BASE_DIR, 'data', 'visualgkg.csv'))
        return self.assertTrue(tabs.equals(_tableinfo('vision',version=2)))

    def testtable_iatv(self):
        """Make the cameo code description"""
        tabs = pd.read_csv(
            os.path.join(BASE_DIR, 'data', 'iatv.csv'))
        return self.assertTrue(tabs.equals(_tableinfo('iatv',version=2)))


    def testtable_cameo(self):
        """Make the cameo code description"""
        tabs = pd.read_json(
            os.path.join(BASE_DIR, 'data', 'cameoCodes.json'),
            dtype={'cameoCode': 'str', "GoldsteinScale": np.float64})

        return self.assertEqual(_tableinfo('cameo').shape,tabs.shape)


    def testtable_fail(self):
        """Make the cameo code description"""
        table = 'bobby'
        valid = ['cameo', 'events', 'gkg', 'vgkg', 'iatv',
                 'graph', 'ments', 'mentions', 'cloudviz',
                 'cloud vision', 'vision']
        exp = ('You entered "{}"; this is not a valid table name.'
                         ' Choose from {}.'.format(
            table,", ".join(valid)))
        with self.assertRaises(Exception) as context:
            checked = _tableinfo('bobby',version=1)
        the_exception = context.exception

        return self.assertEqual(exp, str(the_exception))







#!/usr/bin/python
# -*- coding: utf-8 -*-

# Author:
# Linwood Creekmore
# Email: valinvescap@gmail.com


##################################
# Standard library imports
##################################

import datetime
import json
import multiprocessing.pool
import os
import re
from functools import partial
from multiprocessing import Pool, cpu_count
import concurrent

##################################
# Third party imports
##################################

import numpy as np
import pandas as pd
import requests

##################################
# Local imports
##################################
from gdelt.dateFuncs import (_dateRanger, _gdeltRangeString)
from gdelt.getHeaders import _events1Heads, _events2Heads, _mentionsHeads, \
    _gkgHeads
from gdelt.helpers import _cameos, _tableinfo
from gdelt.inputChecks import (_date_input_check)
from gdelt.parallel import _mp_worker
from gdelt.vectorizingFuncs import _urlBuilder, _geofilter


##################################
# Third party imports
##################################


class NoDaemonProcess(multiprocessing.Process):
    # make 'daemon' attribute always return False
    @property
    def _get_daemon(self):  # pragma: no cover
        return False

    def _set_daemon(self, value):  # pragma: no cover
        pass

    daemon = property(_get_daemon, _set_daemon)

# We sub-class multiprocessing.pool.Pool instead of multiprocessing.Pool
# because the latter is only a wrapper function, not a proper class.
class NoDaemonProcessPool(multiprocessing.pool.Pool):
    Process = NoDaemonProcess


##############################################
#  Admin to load local files
##############################################


this_dir, this_filename = os.path.split(__file__)
BASE_DIR = os.path.dirname(this_dir)

UTIL_FILES_PATH = os.path.join(BASE_DIR, "gdeltPyR", "utils", "schema_csvs")

try:

    codes = pd.read_json(os.path.join(BASE_DIR, 'data', 'cameoCodes.json'),
                         dtype=dict(cameoCode='str', GoldsteinScale=float),precise_float=True,convert_dates=False)
    codes.set_index('cameoCode', drop=False, inplace=True)


except:  # pragma: no cover
    print('here')
    a = 'https://raw.githubusercontent.com/linwoodc3/gdeltPyR/master' \
        '/utils/' \
        'schema_csvs/cameoCodes.json'
    codes = json.loads((requests.get(a).content.decode('utf-8')))

##############################
# Core GDELT class
##############################


class gdelt(object):
    """GDELT Object
        Read more in the :ref:`User Guide <k_means>`.

        Attributes
        ----------
        version : int, optional, {2,1}
            The version of GDELT services used by gdelt. 1 or 2
        gdelt2url : string,default: http://data.gdeltproject.org/gdeltv2/
            Base url for GDELT 2.0 services.
        gdelt1url : string, default: http://data.gdeltproject.org/events/
            Base url for GDELT 1.0 services.
        cores : int, optional, default: system-generated
            Count of total CPU cores available.
        pool: function
            Standard multiprocessing function to establish Pool workers
        proxies: dict
            Dictionary containing proxy information for the requests module.
            For details on how to set, see
            http://docs.python-requests.org/en/master/user/advanced/#proxies
            Example:
                >>>proxies = {'http': 'http://10.10.1.10:3128',\
                'https': 'http://10.10.1.10:1080'}
                >>> requests.get('http://example.org', proxies=proxies)
                Or with a password or specific schema
                >>>proxies = {'http': 'http://user:pass@10.10.1.10:3128/'}
                >>>proxies = {'http://10.20.1.128': 'http://10.10.1.10:5323'}


        # TODO add abiity to pick custom time windows

        Examples
        --------
        >>> from gdelt
        >>> gd = gdelt.gdelt(version=2)
        >>> results = gd.Search(['2016 10 19'],table='events',coverage=True)
        >>> print(len(results))
        244767
        >>> print(results.columns)
        Index(['GLOBALEVENTID', 'SQLDATE', 'MonthYear', 'Year', 'FractionDate',
       'Actor1Code', 'Actor1Name', 'Actor1CountryCode', 'Actor1KnownGroupCode',
       'Actor1EthnicCode', 'Actor1Religion1Code', 'Actor1Religion2Code',
       'Actor1Type1Code', 'Actor1Type2Code', 'Actor1Type3Code', 'Actor2Code',
       'Actor2Name', 'Actor2CountryCode', 'Actor2KnownGroupCode',
       'Actor2EthnicCode', 'Actor2Religion1Code', 'Actor2Religion2Code',
       'Actor2Type1Code', 'Actor2Type2Code', 'Actor2Type3Code', 'IsRootEvent',
       'EventCode', 'EventBaseCode', 'EventRootCode', 'QuadClass',
       'GoldsteinScale', 'NumMentions', 'NumSources', 'NumArticles', 'AvgTone',
       'Actor1Geo_Type', 'Actor1Geo_FullName', 'Actor1Geo_CountryCode',
       'Actor1Geo_ADM1Code', 'Actor1Geo_ADM2Code', 'Actor1Geo_Lat',
       'Actor1Geo_Long', 'Actor1Geo_FeatureID', 'Actor2Geo_Type',
       'Actor2Geo_FullName', 'Actor2Geo_CountryCode', 'Actor2Geo_ADM1Code',
       'Actor2Geo_ADM2Code', 'Actor2Geo_Lat', 'Actor2Geo_Long',
       'Actor2Geo_FeatureID', 'ActionGeo_Type', 'ActionGeo_FullName',
       'ActionGeo_CountryCode', 'ActionGeo_ADM1Code', 'ActionGeo_ADM2Code',
       'ActionGeo_Lat', 'ActionGeo_Long', 'ActionGeo_FeatureID', 'DATEADDED',
       'SOURCEURL'],
       dtype='object')


        Notes
        ------
        gdeltPyR retrieves Global Database of Events, Language, and Tone
        (GDELT) data (version 1.0 or version 2.0) via parallel HTTP GET
        requests and is an alternative to accessing GDELT
        data via Google BigQuery .

        Performance will vary based on the number of available cores
        (i.e. CPUs), internet connection speed, and available RAM. For
        systems with limited RAM, Later iterations of gdeltPyR will include
        an option to store the output directly to disc.
        """

    def __init__(self,
                 gdelt2url='http://data.gdeltproject.org/gdeltv2/',
                 gdelt1url='http://data.gdeltproject.org/events/',
                 version=2.0,
                 cores=cpu_count(),
                 proxies=None

                 ):

        self.codes = codes
        self.translation = None
        self.version = version
        self.cores = cores
        self.proxies = proxies
        if int(version) == 2:
            self.baseUrl = gdelt2url
        elif int(version) == 1:
            self.baseUrl = gdelt1url
        self.proxies = proxies
        if proxies:
            if isinstance(proxies, dict):
                self.proxies = proxies
            else:
                raise TypeError("The proxies parameter must be a dictionary. "
                                "See http://docs.python-requests.org/en/master/"
                                "user/advanced/#proxies for more information.")


    ###############################
    # Searcher function for GDELT
    ###############################

    def Search(self,
               date,
               table='events',
               coverage=False,
               translation=False,
               output=None,
               queryTime=datetime.datetime.now().strftime('%m-%d-%Y %H:%M:%S'),
               normcols=False
               ):
        """Core searcher method to set parameters for GDELT data searches

        Keyword arguments
        ----------
        date : str, required
            The string representation of a datetime (single) or date
            range (list of strings) that is (are) the targeted timelines to
            pull GDELT data.

        table : string,{'events','gkg','mentions'}
            Select from the table formats offered by the GDELT service:

                * events (1.0 and 2.0)

                    The biggest difference between 1.0 and 2.0 are the
                    update frequencies.  1.0 data is disseminated daily,
                    and the most recent data will be published at 6AM
                    Eastern Standard time of the next day. So, 21 August 2016
                    results would be available 22 August 2016 at 6AM EST.  2.0
                    data updates every 15 minutes of the current day.


                    Version 1.0  runs from January 1, 1979 through March 31,
                    2013 contains 57 fields for each record. The Daily
                    Updates  collection, which begins April 1, 2013 and runs
                    through present, contains an additional field at the end
                    of each record, for a total of 58 fields for each
                    record. The format is dyadic CAMEO format, capturing two
                    actors and the action performed by Actor1 upon Actor2.

                    Version 2.0 only covers February 19, 2015 onwards,
                    and is stored in an expanded version of the dyadic CAMEO
                    format .  See
                    http://data.gdeltproject.org/documentation/GDELT-Event_
                    Codebook-V2.0.pdf for more information.

                * gkg  (1.0 and 2.0)

                    **Warning** These tables and queries can be extremely
                    large and consume a lot of RAM. Consider running a
                    single days worth of gkg pulls, store to disc,
                    flush RAM, then proceed to the next day.

                    Table that represents all of the latent dimensions,
                    geography, and network structure of the global news. It
                    applies an array of highly sophisticated natural language
                    processing algorithms to each document to compute a range
                    of codified metadata encoding key latent and contextual
                    dimensions of the document.  Version 2.0 includes Global
                    Content Analysis Measures (GCAM) which reportedly
                    provides 24 emotional measurement packages that assess
                    more than 2,300 emotions and themes from every article
                    in realtime, multilingual  dimensions natively assessing
                    the emotions of 15 languages (Arabic, Basque, Catalan,
                    Chinese, French, Galician, German, Hindi, Indonesian,
                    Korean, Pashto, Portuguese, Russian, Spanish,
                    and Urdu).See documentation about GKG
                    1.0 at http://data.gdeltproject.org/documentation/GDELT-
                    Global_Knowledge_Graph_Codebook.pdf, and GKG 2.0 at http://
                    data.gdeltproject.org/documentation/GDELT-Global_Knowledge_
                    Graph_Codebook-V2.1.pdf.

                * mentions  (2.0 only)

                     Mentions table records every mention
                     of an event over time, along with the timestamp the
                     article was published. This allows the progression of
                     an event   through the global media to be tracked,
                     identifying  outlets that tend to break certain kinds
                     of events the  earliest or which may break stories
                     later but are more  accurate in their reporting on
                     those events. Combined  with the 15 minute update
                     resolution and GCAM, this also  allows the emotional
                     reaction and resonance of an event to be assessed as
                     it sweeps through the world’s media.

        coverage : bool, default: False
            When set to 'True' and the GDELT version parameter is set to 2,
            gdeltPyR will pull back every 15 minute interval in the day (
            full results) or, if pulling for the current day, pull all 15
            minute intervals up to the most recent 15 minute interval of the
            current our.  For example, if the current date is 22 August,
            2016 and the current time is 0828 HRs Eastern, our pull would
            get pull every 15 minute interval in the day up to 0815HRs.
            When coverate is set to true and a date range is entered,
            we pull every 15 minute interval for historical days and up to
            the most recent 15 minute interval for the current day, if that
            day is included.
            
        translation : bool, default: False
            Whether or not to pull the translation database available from
            version 2 of GDELT. If translation is True, the translated set
            is downloaded, if set to False the english set is downloaded. 

        queryTime : datetime object, system generated
            This records the system time when gdeltPyR's query was executed,
            which can be used for logging purposes.

        output : string, {None,'df','gpd','shp','shapefile', 'json', 'geojson'
                'r','geodataframe'}
            Select the output format for the returned GDELT data

            Options
            -------

            json - Javascript Object Notation output; returns list of
            dictionaries in Python or a list of json objects

            r - writes the cross language dataframe to the current directory.
            This uses the Feather library found at https://github.com/wesm/
            feather.  This option returns a pandas dataframe but write the R
            dataframe to the current working directory. The filename
            includes all the parameters used to launch the query: version,
            coverage, table name, query dates, and query time.

            csv- Outputs a CSV format; all dates and columns are joined
            
            shp- Writes an ESRI shapefile to current directory or path; output
            is filtered to exclude rows with no latitude or longitude
            
            geojson- 
            
            geodataframe- Returns a geodataframe; output is filtered to exclude
            rows with no latitude or longitude.  This output can be manipulated
            for geoprocessing/geospatial operations such as reprojecting the 
            coordinates, creating a thematic map (choropleth map), merging with
            other geospatial objects, etc.  See http://geopandas.org/ for info.

        normcols : bool
            Applies a generic lambda function to normalize GDELT columns 
            for compatibility with SQL or Shapefile outputs.  
        Examples
        --------
        >>> from gdelt
        >>> gd = gdelt.gdelt(version=1)
        >>> results = gd.Search(['2016 10 19'],table='events',coverage=True)
        >>> print(len(results))
        244767
        >>> gd = gdelt.gdelt(version=2)
        >>> results = gd.Search(['2016 Oct 10'], table='gkg')
        >>> print(len(results))
        2398
        >>> print(results.V2Persons.iloc[2])
        Juanita Broaddrick,1202;Monica Lewinsky,1612;Donald Trump,12;Donald
        Trump,244;Wolf Blitzer,1728;Lucianne Goldberg,3712;Linda Tripp,3692;
        Bill Clinton,47;Bill Clinton,382;Bill Clinton,563;Bill Clinton,657;Bill
         Clinton,730;Bill Clinton,1280;Bill Clinton,2896;Bill Clinton,3259;Bill
          Clinton,4142;Bill Clinton,4176;Bill Clinton,4342;Ken Starr,2352;Ken
          Starr,2621;Howard Stern,626;Howard Stern,4286;Robin Quivers,4622;
          Paula Jones,3187;Paula Jones,3808;Gennifer Flowers,1594;Neil Cavuto,
          3362;Alicia Machado,1700;Hillary Clinton,294;Hillary Clinton,538;
          Hillary Clinton,808;Hillary Clinton,1802;Hillary Clinton,2303;Hillary
           Clinton,4226
        >>> results = gd.Search(['2016 Oct 10'], table='gkg',output='r')

        Notes
        ------
        Read more about GDELT data at http://gdeltproject.org/data.html

        gdeltPyR retrieves Global Database of Events, Language, and Tone
        (GDELT) data (version 1.0 or version 2.0) via parallel HTTP GET
        requests and is an alternative to accessing GDELT
        data via Google BigQuery.

        Performance will vary based on the number of available cores
        (i.e. CPUs), internet connection speed, and available RAM. For
        systems with limited RAM, Later iterations of gdeltPyR will include
        an option to store the output directly to disc.

        """

        # check for valid table names; fail early
        valid = ['events', 'gkg', 'vgkg', 'iatv', 'mentions']
        if table not in valid:
            raise ValueError('You entered "{}"; this is not a valid table name.'
                             ' Choose from "events", "mentions", or "gkg".'
                .format(table))

        _date_input_check(date, self.version)
        self.coverage = coverage
        self.date = date
        version = self.version
        baseUrl = self.baseUrl
        self.queryTime = queryTime
        self.table = table
        self.translation = translation
        self.datesString = _gdeltRangeString(_dateRanger(self.date),
                                            version=version,
                                            coverage=self.coverage)


        #################################
        # R dataframe check; fail early
        #################################
        if output == 'r':  # pragma: no cover
            try:
                import feather

            except ImportError:
                raise ImportError(('You need to install `feather` in order '
                                   'to output data as an R dataframe. Keep '
                                   'in mind the function will return a '
                                   'pandas dataframe but write the R '
                                   'dataframe to your current working '
                                   'directory as a `.feather` file.  Install '
                                   'by running\npip install feather\nor if '
                                   'you have Anaconda (preferred)\nconda '
                                   'install feather-format -c conda-forge\nTo '
                                   'learn more about the library visit https:/'
                                   '/github.com/wesm/feather'))

        ##################################
        # Partial Functions
        #################################

        v1RangerCoverage = partial(_gdeltRangeString, version=1,
                                   coverage=True)
        v2RangerCoverage = partial(_gdeltRangeString, version=2,
                                   coverage=True)
        v1RangerNoCoverage = partial(_gdeltRangeString, version=1,
                                     coverage=False)
        v2RangerNoCoverage = partial(_gdeltRangeString, version=2,
                                     coverage=False)
        urlsv1gkg = partial(_urlBuilder, version=1, table='gkg')
        urlsv2mentions = partial(_urlBuilder, version=2, table='mentions', translation=self.translation)
        urlsv2events = partial(_urlBuilder, version=2, table='events', translation=self.translation)
        urlsv1events = partial(_urlBuilder, version=1, table='events')
        urlsv2gkg = partial(_urlBuilder, version=2, table='gkg', translation=self.translation)

        eventWork = partial(_mp_worker, table='events', proxies=self.proxies)
        codeCams = partial(_cameos, codes=self.codes)

        #####################################
        # GDELT Version 2.0 Headers
        #####################################

        if int(self.version) == 2:
            ###################################
            # Download 2.0 Headers
            ###################################

            if self.table =='events':
                try:
                    self.events_columns = \
                    pd.read_csv(os.path.join(BASE_DIR, "data", 'events2.csv'))[
                        'name'].values.tolist()

                except:  # pragma: no cover
                    self.events_columns = _events2Heads()

            elif self.table == 'mentions':
                try:
                    self.mentions_columns = \
                        pd.read_csv(
                            os.path.join(BASE_DIR, "data", 'mentions.csv'))[
                            'name'].values.tolist()

                except:  # pragma: no cover
                    self.mentions_columns = _mentionsHeads()
            else:
                try:
                    self.gkg_columns = \
                        pd.read_csv(
                            os.path.join(BASE_DIR, "data", 'gkg2.csv'))[
                            'name'].values.tolist()

                except:  # pragma: no cover
                    self.gkg_columns = _gkgHeads()

        #####################################
        # GDELT Version 1.0 Analytics, Header, Downloads
        #####################################

        if int(self.version) == 1:

            if self.table == "mentions":
                raise ValueError('GDELT 1.0 does not have the "mentions"'
                                    ' table. Specify the "events" or "gkg"'
                                    'table.')
            if self.translation:
                raise ValueError('GDELT 1.0 does not have an option to'
                                    ' return translated table data. Switch to '
                                    'version 2 by reinstantiating the gdelt '
                                    'object with <gd = gdelt.gdelt(version=2)>')
            else:
                pass

            try:
                self.events_columns = \
                    pd.read_csv(os.path.join(BASE_DIR, "data", 'events1.csv'))[
                        'name'].values.tolist()

            except:  # pragma: no cover
                self.events_columns = _events1Heads()

            columns = self.events_columns

            if self.table == 'gkg':
                self.download_list = (urlsv1gkg(v1RangerCoverage(
                    _dateRanger(self.date))))

            elif self.table == 'events' or self.table == '':

                if self.coverage is True:  # pragma: no cover

                    self.download_list = (urlsv1events(v1RangerCoverage(
                        _dateRanger(self.date))))

                else:
                    # print("I'm here at line 125")
                    self.download_list = (urlsv1events(v1RangerNoCoverage(
                        _dateRanger(self.date))))

            else:  # pragma: no cover
                raise Exception('You entered an incorrect table type for '
                                'GDELT 1.0.')
        #####################################
        # GDELT Version 2.0 Analytics and Download
        #####################################
        elif self.version == 2:

            if self.table == 'events' or self.table == '':
                columns = self.events_columns
                if self.coverage is True:  # pragma: no cover

                    self.download_list = (urlsv2events(v2RangerCoverage(
                        _dateRanger(self.date))))
                else:

                    self.download_list = (urlsv2events(v2RangerNoCoverage(
                        _dateRanger(self.date))))

            if self.table == 'gkg':
                columns = self.gkg_columns
                if self.coverage is True:  # pragma: no cover

                    self.download_list = (urlsv2gkg(v2RangerCoverage(
                        _dateRanger(self.date))))
                else:
                    self.download_list = (urlsv2gkg(v2RangerNoCoverage(
                        _dateRanger(self.date))))
                    # print ("2 gkg", urlsv2gkg(self.datesString))

            if self.table == 'mentions':
                columns = self.mentions_columns
                if self.coverage is True:  # pragma: no cover

                    self.download_list = (urlsv2mentions(v2RangerCoverage(
                        _dateRanger(self.date))))

                else:

                    self.download_list = (urlsv2mentions(v2RangerNoCoverage(
                        _dateRanger(self.date))))


        #########################
        # DEBUG Print Section
        #########################


        # if isinstance(self.datesString,str):
        #     if parse(self.datesString) < datetime.datetime.now():
        #         self.datesString = (self.datesString[:8]+"234500")
        # elif isinstance(self.datesString,list):
        #     print("it's a list")
        # elif isinstance(self.datesString,np.ndarray):
        #     print("it's an array")
        # else:
        #     print("don't know what it is")
        # print (self.version,self.download_list,self.date, self.table, self.coverage, self.datesString)
        #
        # print (self.download_list)
        # if self.coverage:
        #     coverage = 'True'
        # else:
        #     coverage = 'False'
        # if isinstance(self.date, list):
        #
        #     formattedDates = ["".join(re.split(' |-|;|:', l)) for l in
        #                       self.date]
        #     path = formattedDates
        #     print("gdeltVersion_" + str(self.version) +
        #           "_coverage_" + coverage + "_" +
        #           "_table_" + self.table + '_queryDates_' +
        #           "_".join(path) +
        #           "_queryTime_" +
        #           datetime.datetime.now().strftime('%m-%d-%YT%H%M%S'))
        # else:
        #     print("gdeltVersion_" + str(self.version) +
        #           "_coverage_" + coverage + "_" +
        #           "_table_" + self.table + '_queryDates_' +
        #           "".join(re.split(' |-|;|:', self.date)) +
        #           "_queryTime_" +
        #           datetime.datetime.now().strftime('%m-%d-%YT%H%M%S'))

        #########################
        # Download section
        #########################
        # print(self.download_list,type(self.download_list))

        # from gdelt.extractors import normalpull
        # e=ProcessPoolExecutor()
        # if isinstance(self.download_list,list) and len(self.download_list)==1:
        #     from gdelt.extractors import normalpull
        #
        #     results=normalpull(self.download_list[0],table=self.table)
        # elif isinstance(self.download_list,list):
        #     print(table)
        #     multilist = list(e.map(normalpull,self.download_list))
        #     results = pd.concat(multilist)
        # print(results.head())

        if isinstance(self.datesString, str):
            if self.table == 'events':

                results = eventWork(self.download_list)
            else:
                # if self.table =='gkg':
                #     results = eventWork(self.download_list)
                #
                # else:
                results = _mp_worker(self.download_list, proxies=self.proxies)

        else:

            if self.table == 'events':

                pool = Pool(processes=cpu_count())
                downloaded_dfs = list(pool.imap_unordered(eventWork,
                                                          self.download_list))
                pool.close()
                pool.terminate()
                pool.join()
            else:

                #pool = NoDaemonProcessPool(processes=cpu_count())
                with concurrent.futures.ProcessPoolExecutor() as executor:
                    # Submit tasks to the executor
                    downloaded_dfs = list(executor.map(_mp_worker,self.download_list))

                    # downloaded_dfs = []
                    #
                    # # Wait for all of the tasks to finish executing
                    # for result in results:
                    #     downloaded_dfs = downloaded_dfs.append(results)
                # downloaded_dfs = list(pool.imap_unordered(_mp_worker,
                #                                           self.download_list,
                #                                           ))

            # print(downloaded_dfs)
            results = pd.concat(downloaded_dfs)
            del downloaded_dfs
            results.reset_index(drop=True, inplace=True)


        if self.table == 'gkg' and self.version == 1:
            results.columns = results.iloc[0].values.tolist()
            results.drop([0], inplace=True)
            columns = results.columns

        # check for empty dataframe
        if results is not None:
            if len(results.columns) == 57:  # pragma: no cover
                results.columns = columns[:-1]

            else:
                results.columns = columns

        # if dataframe is empty, raise error
        elif results is None or len(results) == 0:  # pragma: no cover
            raise ValueError("This GDELT query returned no data. Check "
                             "query parameters and "
                             "retry")

        # Add column of human readable codes; need updated CAMEO
        if self.table == 'events':
            cameoDescripts = results.EventCode.apply(codeCams)

            results.insert(27, 'CAMEOCodeDescription',
                           value=cameoDescripts.values)

        ###############################################
        # Setting the output options
        ###############################################

        # dataframe output
        if output == 'df':
            self.final = results

        # json output
        elif output == 'json':
            self.final = results.to_json(orient='records')

        # csv output
        elif output == 'csv':
            self.final = results.to_csv(encoding='utf-8')

        # geopandas dataframe output
        elif output == 'gpd' or output == 'geodataframe' or output == 'geoframe':
            self.final = _geofilter(results)
            self.final = self.final[self.final.geometry.notnull()]

        # r dataframe output
        elif output == 'r':  # pragma: no cover
            if self.coverage:
                coverage = 'True'
            else:
                coverage = 'False'
            if isinstance(self.date, list):

                formattedDates = ["".join(re.split(' |-|;|:', l)) for l in
                                  self.date]
                path = formattedDates
                outPath = ("gdeltVersion_" + str(self.version) +
                           "_coverage_" + coverage + "_" +
                           "_table_" + self.table + '_queryDates_' +
                           "_".join(path) +
                           "_queryTime_" +
                           datetime.datetime.now().strftime('%m-%d-%YT%H%M%S') +
                           ".feather")
            else:
                outPath = ("gdeltVersion_" + str(self.version) +
                           "_coverage_" + coverage + "_" +
                           "_table_" + self.table + '_queryDates_' +
                           "".join(re.split(' |-|;|:', self.date)) +
                           "_queryTime_" +
                           datetime.datetime.now().strftime('%m-%d-%YT%H%M%S') +
                           ".feather")

            if normcols:
                results.columns = list(map(lambda x: (x.replace('_', "")).lower(), results.columns))

            feather.api.write_dataframe(results, outPath)
            return results

        else:
            self.final = results

        #########################
        # Return the result
        #########################

        # normalized columns
        if normcols:
            self.final.columns = list(map(lambda x: (x.replace('_', "")).lower(), self.final.columns))

        return self.final


    def schema(self,tablename):
        """

        Parameters
        ----------
        :param tablename: str
            Name of table to retrieve desired schema

        Returns
        -------
        :return: dataframe
            pandas dataframe with schema
        """

        return _tableinfo(table=tablename,
                          version=self.version)  # pragma: no cover



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Author:
# Linwood Creekmore
# Email: valinvescap@gmail.com

##################################
# Standard library imports
##################################
import argparse
from multiprocessing import cpu_count

##################################
# Third party imports
##################################
import numpy as np
import pandas as pd
from concurrent.futures import ProcessPoolExecutor

##################################
# Local imports
##################################
from gdelt.helpers import _shaper

cores = cpu_count()
e = ProcessPoolExecutor(max_workers=cores)


# def _shaper(row):
#     """
#     Parallel function to create shapely points
#     from latitude/longitude pair in dataframe
#
#     Parameters
#     ----------
#     row : pandas or dask dataframe row
#         Row containing latitude and longitude variables and data
#
#     Returns
#     -------
#     shapely point
#         Shapely spatial object for geoprocessing in Python.
#     """
#     try:
#         from shapely.geometry import Point
#     except:
#         raise ImportError('You need to install shapely to use this feature.')
#
#     try:
#         import fiona
#     except:
#         raise ImportError('You need to install fiona to use this feature.')
#     try:
#         import geopandas
#     except:
#         raise ImportError('You need to install geopandas to use this feature.')
#     geometry = Point(row['ActionGeo_Long'], row['ActionGeo_Lat'])
#     return geometry


def _call_apply_fn(df):
    return df.apply(_shaper, axis=1)


def _parallelize_dataframe(df):
    """Applying function"""
    df_split = np.array_split(df, cores * 2)
    finaldf = pd.concat(list(e.map(_call_apply_fn, df_split)))
    return finaldf


#
# if __name__ == '__main__':
#     import sys
#     parallelize_dataframe(sys.argv[1:])
if __name__ == '__main__':  # pragma: no cover
    parser = argparse.ArgumentParser(description='Parallelize pandas apply function')
    parser.add_argument('--df', metavar='pandas.core.frame.DataFrame', required=True,
                        help='The target dataframe to apply the function.')
    parser.add_argument('--func', metavar='function', required=False,
                        help='The function to be applied to the dataframe.')
    args = parser.parse_args()
    # model_schema(df=args.df, func=args.func)



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Author:
# Linwood Creekmore
# Email: valinvescap@gmail.com

##############################
# Standard Library Import
##############################

try:
    from unittest import TestCase, mock
except:
    from unittest import TestCase
    import mock

import os
import sys
import warnings
from io import BytesIO, StringIO
from unittest import TestCase
from zipfile import ZipFile

import numpy as np
import pandas as pd

import gdelt
from gdelt.parallel import _mp_worker


##############################
# Third Party Libraries
##############################
##############################
# Custom Library Import
##############################


class testParallelWorker(TestCase):



    @mock.patch.object(gdelt.parallel.requests, 'get')
    def test_parallel_events2_pass(self, mock_B):
        """Return csv."""

        # set up return value for mock
        ver = sys.version_info.major
        spam = pd.read_pickle(os.path.join(
                gdelt.base.BASE_DIR, "data", "events2samp.gz"),
                compression="gzip").drop('CAMEOCodeDescription',axis=1)

        spam.columns = np.arange(len(spam.columns))
        spam[[26,27,28]] = spam[[26,27,28]].astype('str')

        if ver == 3:
            buffer = StringIO()
            spam.to_csv(buffer, sep='\t', header=False,index=False)
        else:
            buffer = BytesIO()
            spam.to_csv(buffer, sep='\t', header=False, index=False)
        # print(spam)

        # making the zipfile
        inMemoryOutputFile = BytesIO()
        zipFile = ZipFile(inMemoryOutputFile, 'w')
        zipFile.writestr('OEBPS/20170701234500.export.CSV.zip',
                         buffer.getvalue())
        zipFile.close()

        # make the mock object
        response = mock_B()
        response.status_code=200
        response.content = inMemoryOutputFile.getvalue()
        response.return_value = inMemoryOutputFile

        # run the function of gdelt
        url = 'http://data.gdeltproject.org/gdeltv2/20170701234500.export.CSV.zip'
        res = _mp_worker(url,table='events')
        print(res.columns)

        # the test
        return self.assertTrue(res[[0,1,2,3]].equals(spam[[0,1,2,3]]), "Returned dataframe")

    @mock.patch.object(gdelt.parallel.requests, 'get')
    def test_parallel_404_warnings(self, mock_B):
        """Return csv."""

        # make the mock object
        response = mock_B()
        response.status_code=404

        # run the function of gdelt
        url = 'http://data.gdeltproject.org/gdeltv2/20170727234500.export.CSV.zip'
        exp = [("GDELT does not have a url for date time 20170727234500"),
               ("GDELT did not return data for date time 20170727234500")]

        with warnings.catch_warnings(record=True) as w:
            # Cause all warnings to always be triggered.
            warnings.simplefilter("always")
            # Trigger a warning.
            res = _mp_worker(url, table='events')
            # Verify some things
            # print(str(w[0].message),str(w[1].message))
            # assert len(w) == 1
            # assert issubclass(w[-1].category, UserWarning)
            # assert "deprecated" in str(w[-1].message)

        # the test
        return self.assertEqual([str(w[0].message),str(w[1].message)],exp, "Returned dataframe")

    @mock.patch.object(gdelt.parallel.requests, 'get')
    def test_parallel_gkgv2_pass(self, mock_B):
        """Return csv."""

        # set up return value for mock
        spam = pd.read_pickle(os.path.join(
            gdelt.base.BASE_DIR, "data", "gkg2samp.gz"),
            compression="gzip")
        spam.columns = np.arange(len(spam.columns))
        ver = sys.version_info.major
        if ver == 3:
            buffer = StringIO()
            spam.to_csv(buffer, sep='\t', header=False,index=False)
        else:
            buffer = BytesIO()
            spam.to_csv(buffer, sep='\t', header=False, index=False,encoding='utf-8')

        # print(spam)

        # making the zipfile
        inMemoryOutputFile = BytesIO()
        zipFile = ZipFile(inMemoryOutputFile, 'w')
        zipFile.writestr('OEBPS/20170701234500.gkg.csv.zip',
                         buffer.getvalue())
        zipFile.close()

        # make the mock object
        response = mock_B()
        response.status_code = 200
        response.content = inMemoryOutputFile.getvalue()
        response.return_value = inMemoryOutputFile

        # run the function of gdelt
        url = 'http://data.gdeltproject.org/gdeltv2/20170701234500.gkg.csv.zip'
        res = _mp_worker(url, table='gkg')

        # the test
        return self.assertTrue(res[[0, 1, 2, 3]].equals(spam[[0, 1, 2, 3]]),
                               "Returned dataframe")

    @mock.patch.object(gdelt.parallel.requests, 'get')
    def test_parallel_mentions_pass(self, mock_B):
        """Return csv."""

        # set up return value for mock
        spam = pd.read_pickle(os.path.join(
            gdelt.base.BASE_DIR, "data", "mentionssamp.gz"),
            compression="gzip")
        spam.columns = np.arange(len(spam.columns))
        ver = sys.version_info.major
        if ver == 3:
            buffer = StringIO()
            spam.to_csv(buffer, sep='\t', header=False, index=False)
        else:
            buffer = BytesIO()
            spam.to_csv(buffer, sep='\t', header=False, index=False,
                        encoding='utf-8')
        # print(spam)

        # making the zipfile
        inMemoryOutputFile = BytesIO()
        zipFile = ZipFile(inMemoryOutputFile, 'w')
        zipFile.writestr('OEBPS/20170701234500.mentions.CSV.zip',
                         buffer.getvalue())
        zipFile.close()

        # make the mock object
        response = mock_B()
        response.status_code = 200
        response.content = inMemoryOutputFile.getvalue()
        response.return_value = inMemoryOutputFile

        # run the function of gdelt
        url = ('http://data.gdeltproject.org/gdeltv2/'
               '20170701234500.mentions.CSV.zip')
        res = _mp_worker(url, table='gkg')

        # the test
        return self.assertTrue(res[[0, 1, 2, 3]].equals(spam[[0, 1, 2, 3]]),
                               "Returned dataframe")


#!/usr/bin/python
# -*- coding: utf-8 -*-

# Author:
# Linwood Creekmore
# Email: valinvescap@gmail.com

##############################
# Standard Library imports
##############################

import datetime
from unittest import TestCase

import numpy as np
from dateutil.parser import parse

from gdelt.inputChecks import _date_input_check


##############################
# Third party imports
##############################
##############################
# Local imports
##############################

class TestGdeltDateInputs(TestCase):
    def test_gdeltdate_check_v2_fail(self):
        date_sequence = ['2011 10 01', '2016 10 05']
        exp = "GDELT 2.0 only supports 'Feb 18 2015 - Present'queries " \
              "currently. Try another date."
        with self.assertRaises(Exception) as context:
            checked = _date_input_check(date_sequence, version=2)
        the_exception = context.exception
        return self.assertEqual(exp, str(the_exception), "Dates for version 2 "
                                                         "fail outside of "
                                                         "Feb 18 2015")

    def test_gdeltdate_check_v2_fail_firstgreater(self):
        date_sequence = ['2017 Feb 01','2016 Feb 29']
        exp = ("Start date greater than or equal to end date. Check your "
               "entered date query.")
        with self.assertRaises(Exception) as context:
            checked = _date_input_check(date_sequence, version=2)
        the_exception = context.exception

        return self.assertEqual(exp, str(the_exception), "Dates for version"
                                                         " 2 fail outside of "
                                                         "Feb 18 2015")

    def test_gdeltdate_check_v1_fail_current(self):
        date_sequence = str(datetime.datetime.now().date())
        exp = ("You entered today's date for a GDELT 1.0 query. GDELT 1.0's "
               "most recent data is always the trailing day (i.e. "
               "2017-08-11).  Please retry your query.")

        with self.assertRaises(Exception) as context:
            checked = _date_input_check(date_sequence, version=1)
        the_exception = context.exception

        return self.assertIsInstance(str(the_exception),str, "Too early date "
                                                         "on Version 1")
    def test_gdeltdate_check_v2_fail_one_wrong_inlist(self):
        date_sequence = np.asarray(['2016 Jan 2', '2016 Jan 31',
                                    str(datetime.timedelta(days=1) +
                                        datetime.datetime.now())[:10]])
        exp = ("One of your dates is greater than the current date. "
                    "Check your entered date query.")

        with self.assertRaises(Exception) as context:
            checked = _date_input_check(date_sequence, version=2)
        the_exception = context.exception
        print (str(the_exception))

        return self.assertIsInstance(str(the_exception),str, "One wrong date"
                                     " in list of dates greater than 3 failed.")

    def test_gdeltdate_greater_current(self):
        date_sequence = str(datetime.datetime.now().date() + datetime. \
                            timedelta(days=1))
        exp = ('Your date is greater than the current date. '
               'Please enter a relevant date.')
        with self.assertRaises(Exception) as context:
            checked = _date_input_check(date_sequence, version=2)
        the_exception = context.exception
        return self.assertEqual(exp, str(the_exception), "GDELT can't pull "
                                                         "from the future "
                                                         "error.")

    def test_gdeltdate_old(self):
        date_sequence = str(
            parse('1999 Jan 12').date())

        checked = _date_input_check(date_sequence, version=1)
        return self.assertIsNone(checked, "GDELT can't pull from the "
                                          "future error.")

    def test_gdeltdate_veryold(self):
        date_sequence = ['1982', '1983', '1984'
                         ]
        exp = ('Your date is greater than the current date. '
               'Please enter a relevant date.')

        checked = _date_input_check(date_sequence, version=1)
        return self.assertIsNone(checked, "GDELT can't pull from the "
                                          "future error.")

    def test_gdeltdate_second_greater(self):
        date_sequence = ['2017 Jun 20', '1989 Feb 1']
        exp = ('Start date greater than or equal to end date. '
               'Check your entered date query.')
        with self.assertRaises(Exception) as context:
            checked = _date_input_check(date_sequence, version=1)
        the_exception = context.exception
        return self.assertEqual(exp, str(the_exception), "Second greater")

    def test_gdeltdate_greater_currentlist(self):
        date_sequence = [(str(datetime.datetime.now().date() + datetime. \
                              timedelta(days=1)))]

        exp = ("One or more of your input date strings does not parse to "
               "a date format. Check input.")
        with self.assertRaises(Exception) as context:
            checked = _date_input_check(date_sequence, version=2)
        the_exception = context.exception

        return self.assertEqual(exp, str(the_exception))

    def test_gdeltdate_6am_list(self):
        date_sequence = list(map(lambda x: str(parse(x)),
                                 ['2014 Jun 13', '2016 Feb 1', '2017 Jan 22']))

        exp = ("One or more of your input date strings does not parse to "
               "a date format. Check input.")

        checked = _date_input_check(date_sequence, version=1)

        return self.assertIsNone(checked)

    def test_gdeltdate_greater_current(self):
        holder = (datetime.datetime.now()+datetime.timedelta(days=2))
        date_sequence = list(map(lambda x: str(parse(x)),
                                 ['2016 Feb 1', '{}'.format(str(holder))]))

        exp = ("One of your dates is greater than the current date. Check "
               "your entered date query.")

        with self.assertRaises(Exception) as context:
            checked = _date_input_check(date_sequence, version=2)
        the_exception = context.exception

        return self.assertEqual(exp,str(the_exception),'Not greater')


    def test_gdeltdate_two_v2_tooearly(self):

        date_sequence = np.asarray(['2016 Jan 2', '2013 Jan 31','2015 Jan 1'])

        exp = ("GDELT 2.0 only supports \'Feb 18 2015 - Present\'queries "
               "currently. Try another date.")

        with self.assertRaises(Exception) as context:
            checked = _date_input_check(date_sequence, version=2)
        the_exception = context.exception

        return self.assertEqual(exp,str(the_exception),'Not greater')

    # @mock.patch.object(gdelt.inputChecks._date_input_check.datetime.datetime, 'now')
    # def test_gdeltdate_6am_v1(self):
    #     holder = (datetime.datetime.now()+datetime.timedelta(days=2))
    #     date_sequence = list(map(lambda x: str(parse(x)),
    #                              ['2016 Feb 1', '{}'.format(str(holder))]))
    #
    #     exp = ("One of your dates is greater than the current date. Check "
    #            "your entered date query.")
    #
    #     with self.assertRaises(Exception) as context:
    #         checked = _date_input_check(date_sequence, version=2)
    #     the_exception = context.exception
    #
    #     return self.assertEqual(exp,str(the_exception),'Not greater')





#!/usr/bin/env python

"""A library that provides a Python interface to the GDELT 1.0 and 2.0 API"""
from __future__ import absolute_import

from gdelt.base import gdelt

__name__ = 'gdelt'
__author__ = 'Linwood Creekmore III'
__email__ = 'valinvescap@gmail.com'
__license__ = 'GNU General Public License v3.0'
__version__ = '0.1.14'
__url__ = 'https://linwoodc3.github.io/gdeltPyR/'
__description__ = 'Python based framework to retreive Global Database of ' \
                  'Events, Language, and Tone (GDELT)' \
                  'data in Python Pandas or R dataframes.'



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Author:
# Linwood Creekmore
# Email: valinvescap@gmail.com


##################################
# Standard library imports
##################################

try:
    from unittest import TestCase, mock
except:
    from unittest import TestCase
    import mock

try:
    from urllib.parse import urljoin
except ImportError:
    from urlparse import urljoin
import os

import pandas as pd

import gdelt
from tests.constants import BASEURL

##################################
# Third party imports
##################################
##################################
# Local imports
##################################


this_dir, this_filename = os.path.split(__file__)
BASE_DIR = os.path.dirname(this_dir)


##################################
# Unit Tests
##################################

gdelt1 = urljoin(BASEURL, 'events/index.html')
gdelt2 = urljoin(BASEURL, 'gdeltv2/masterfilelist.txt')
gd2 = gdelt.gdelt(version=2)
gd1 = gdelt.gdelt(version=1)

class testresponse(TestCase):

    @mock.patch('gdelt.gdelt.Search')
    def test_request_response_version2_events(self,mock_get):
        """Send a request to the API server and store the response."""
        gd2 = gdelt.gdelt(version=2)


        # loading the mock dataframe return
        events2 = pd.read_pickle(os.path.join(
            gdelt.base.BASE_DIR, "data",
            "events2samp.gz"),
            compression="gzip")

        # Creating the mock object
        mock_get.return_value = mock.Mock()

        # Adding attributes for the mock; setting it to the pickled dataframe
        mock_get.return_value.dataframe = events2

        # real response for testing
        response = gd2.Search('2017 Jul 1',table='events',coverage=False)

        # Use dataframe method to test for equality
        return (self.assertTrue(response.dataframe.equals(events2)))

    @mock.patch('gdelt.gdelt.Search')
    def test_request_response_version1_events(self,mock_get):
        """Send a request to the API server and store the response."""

        gd1 = gdelt.gdelt(version=1)

        # loading the mock dataframe return

        events1 = pd.read_pickle(os.path.join(
                gdelt.base.BASE_DIR, "data",
                "events1samp.gz"),
                compression="gzip")


        # Creating the mock object
        mock_get.return_value = mock.Mock()

        # Adding attributes for the mock; setting it to the pickled dataframe
        mock_get.return_value.dataframe = events1

        # real response for testing
        response = gd1.Search('2017 Jul 1', table='events', coverage=False)

        # Use dataframe method to test for equality
        return (self.assertTrue(response.dataframe.equals(events1)))




#!/usr/bin/python
# -*- coding: utf-8 -*-

# Author:
# Linwood Creekmore
# Email: valinvescap@gmail.com

##################################
# Standard library imports
##################################
from io import BytesIO

##################################
# Third party imports
##################################
import pandas as pd
import requests


#######################################
# Headers for GDELT 1.0 Events
#######################################



def _events1Heads():
    """

    :rtype: dateframe
    """

    conte = requests.get(
                ("https://raw.githubusercontent.com/linwoodc3/gdeltPyR"
                 "/master/utils/schema_csvs/"
                 "GDELT_1.0_event_Column_Labels_Header_Row_Sep2016.tsv")
            )
    data = BytesIO(conte.content)
    eventsDbHeaders = pd.read_csv(data, delimiter='\t', header=0)

    return eventsDbHeaders[eventsDbHeaders.columns[0]].tolist()


#######################################
# Headers for GDELT 2.0 Events
#######################################

def _events2Heads():
    conte = requests.get(
        ("https://raw.githubusercontent.com/linwoodc3/gdeltPyR/"
         "master/utils/schema_csvs/"
         "GDELT_2.0_Events_Column_Labels_Header_Row_Sep2016.csv")
    )
    data = BytesIO(conte.content)
    eventsDbHeaders = pd.read_csv(data)

    return eventsDbHeaders['tableId'].tolist()


#######################################
# Headers for GDELT 2.0 Mentions
#######################################

def _mentionsHeads():
    conte = requests.get(
        ("https://raw.githubusercontent.com/linwoodc3/gdeltPyR/master/"
         "utils/schema_csvs/"
         "GDELT_2.0_eventMentions_Column_Labels_Header_"
         "Row_Sep2016.tsv")
    )
    data = BytesIO(conte.content)

    eventsDbHeaders = pd.read_csv(data, delimiter='\t',
                                  )
    return eventsDbHeaders['tableId'].tolist()


#######################################
# Headers for GDELT 2.0 GKG
#######################################



def _gkgHeads():

    conte = requests.get(
                ("https://raw.githubusercontent.com/linwoodc3/gdeltPyR/"
                 "master/utils/schema_csvs/"
                 "GDELT_2.0_gdeltKnowledgeGraph_Column_Labels_Header_Row_"
                 "Sep2016.tsv")
            )

    data = BytesIO(conte.content)

    gkgHeaders = pd.read_csv(data
        , delimiter='\t', usecols=['tableId', 'dataType',
                                                 'Description'])
    gkgHeaders.columns = ['tableId', 'dataType',
                               'Description']
    return gkgHeaders.tableId.tolist()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Author:
# Linwood Creekmore
# Email: valinvescap@gmail.com

import datetime
import os
##############################
# Standard Library Imports
##############################
import re

##############################
# Third party Library Imports
##############################

import numpy as np
import pandas as pd
from dateutil.parser import parse

##############################
# Filter functions for dataframes
##############################

this_dir, this_filename = os.path.split(__file__)
BASE_DIR = os.path.dirname(this_dir)

def _cameos(x, codes):
    try:
        return codes['Description'][x]
    except:
        return "No Description returned for CAMEO code {0}".format(x)


def _shaper(row):
    """
    Parallel function to create shapely points
    from latitude/longitude pair in dataframe

    Parameters
    ----------
    row : pandas or dask dataframe row
        Row containing latitude and longitude variables and data

    Returns
    -------
    shapely point
        Shapely spatial object for geoprocessing in Python.
    """
    try:
        from shapely.geometry import Point
    except:
        raise ImportError('You need to install shapely to use this feature.')

    try:
        import fiona
    except:
        raise ImportError('You need to install fiona to use this feature.')
    try:
        import geopandas
    except:
        raise ImportError('You need to install geopandas to use this feature.')
    try:
        geometry = Point(row['ActionGeo_Long'], row['ActionGeo_Lat'])
    except:
        geometry = Point(row['actiongeolong'], row['actiongeolat'])
    return geometry


def _rooturl(row):
    """Finds the root url of a news article"""
    s = row['SOURCEURL']
    r = re.compile('(?<=http://)([A-Za-z0-9.-]+)(?=/)')
    r2 = re.compile('(?<=https://)([A-Za-z0-9.-]+)(?=/)')

    if r.search(s):
        return r.search(s).group()
    elif r2.search(s):
        return r2.search(s).group()

def _testdate(dateString):
    """Test dates of different formats"""
    if len(dateString) == 4:
        comp = datetime.datetime.strptime(dateString, '%Y')
    elif len(dateString) == 6:
        comp = datetime.datetime.strptime(dateString, '%Y%m')
    else:
        comp = parse(dateString)
    return comp


def _tableinfo(table='cameo',version=2):
    """
    Parameters
    -----------
    :param table: str
        Name of GDELT table

    Returns
    --------

    dataframe
        Pandas dataframe containing schema of table

    Examples
    --------
    Example of retrieving the events table schema.

    >>> print([i for i in example_generator(4)])
    [0, 1, 2, 3]
    """



    table = table.lower()

    valid = ['cameo', 'events', 'gkg', 'vgkg', 'iatv',
             'graph', 'ments', 'mentions', 'cloudviz',
             'cloud vision', 'vision']
    if table not in valid:
        raise ValueError('You entered "{}"; this is not a valid table name.'
                         ' Choose from {}.'.format(
            table,", ".join(valid)))

    if table == 'cameo':
        tabs = pd.read_json(
            os.path.join(BASE_DIR, 'data', 'cameoCodes.json'),
            dtype={'cameoCode': 'str', "GoldsteinScale": np.float64})
        tabs.set_index('cameoCode', drop=False, inplace=True)
    elif table == 'events' and float(version) == 1.0:
        tabs = pd.read_csv(os.path.join(BASE_DIR, 'data', 'events1.csv'))

    elif table == 'events' and float(version) == 2.0:
        tabs = pd.read_csv(os.path.join(BASE_DIR, 'data', 'events2.csv'))

    elif table == 'gkg' or table == 'graph':
        tabs = pd.read_csv(os.path.join(BASE_DIR, 'data', 'gkg2.csv'))
    elif table == 'mentions' or table == 'ments':
        if float(version) != 2.0:
            raise ValueError('GDELT 1.0 does not have a mentions table.')
        else:
            tabs = pd.read_csv(
                os.path.join(BASE_DIR, 'data', 'mentions.csv'))

    elif table == 'cloud vision' or table == 'vgkg' or table == 'cloudviz' \
            or table == 'vision':
        tabs = pd.read_csv(os.path.join(BASE_DIR, 'data', 'visualgkg.csv'))

    elif table == 'iatv' or table == 'tv':
        tabs = pd.read_csv(os.path.join(BASE_DIR, 'data', 'iatv.csv'))

    return tabs


# #!/usr/bin/python
# # -*- coding: utf-8 -*-
#
# # Author:
# # Linwood Creekmore
# # Email: valinvescap@gmail.com
#
# ##################################
# # Standard library imports
# ##################################
# import re
# import zipfile
# from io import BytesIO
# import warnings, datetime
#
# ##################################
# # Third party imports
# ##################################
# import pandas as pd
# import requests
#
# ##################################
# # Local imports
# ##################################
#
# def _downloadAndExtract(gdeltUrl):
#     """Downloads and extracts GDELT zips without saving to disk"""
#
#     response = requests.get(gdeltUrl, stream=True)
#     zipdata = BytesIO()
#     zipdata.write(response.content)
#     gdelt_zipfile = zipfile.ZipFile(zipdata, 'r')
#     name = re.search('(([\d]{4,}).*)',
#                      gdelt_zipfile.namelist()[0]).group().replace('.zip', "")
#     data = gdelt_zipfile.read(name)
#     gdelt_zipfile.close()
#     del zipdata, gdelt_zipfile, name, response
#     return pd.read_csv(BytesIO(data), delimiter='\t', header=None)
#
#
# def _normalpull(url,table=None):
#     """When single string url, just download it"""
#     r = requests.get(url, timeout=5)
#     # print(len(r.content))
#     r.raise_for_status()
#
#     # print (multiprocessing.Process(name=multiprocessing.current_process().name).is_alive())
#     try:
#         buffer = BytesIO(r.content)
#         # print(buffer.seek())
#
#         if table == 'events':
#             frame = pd.read_csv(buffer, compression='zip', sep='\t',
#                                 header=None, warn_bad_lines=False,
#                                 dtype={26: 'str', 27: 'str', 28: 'str'},
#                                 parse_dates=[1, 2])
#             # print(frame.shape)
#         elif table == 'gkg':
#             frame = pd.read_csv(buffer, compression='zip', sep='\t',
#                                 parse_dates=['DATE'], warn_bad_lines=False)
#
#         else:
#
#             frame = pd.read_csv(buffer, compression='zip', sep='\t',
#                                 header=None, warn_bad_lines=False)
#         end = datetime.datetime.now() - start
#         # print ("{0} with id {1} finished processing in {2}".format(proc_name,proc,end))
#         buffer.flush()
#         buffer.close()
#         # print(frame.shape)
#         return frame
#
#     except:
#         try:
#             message = "GDELT did not return data for date time " \
#                       "{0}".format(re.search('[0-9]{4,18}', url).group())
#             warnings.warn(message)
#         except:
#             message = "No data returned for {0}".format(r.url)
#             warnings.warn(message)


#!/usr/bin/python
# -*- coding: utf-8 -*-

# Author:
# Linwood Creekmore
# Email: valinvescap@gmail.com

#########################
# Standard Library Import
#########################

from unittest import TestCase

from gdelt.dateFuncs import _dateformatter


############################
# Custom Import
############################

class TestDateFormatter(TestCase):
    def test_dateFormatter(self):
        date_string = 'March 22 2016'
        formatter_test = _dateformatter(date_string)
        exp = '2016-03-22'
        self.assertEqual(formatter_test, exp, "_dateformatter working")




#!/usr/bin/python
# -*- coding: utf-8 -*-

# Author:
# Linwood Creekmore
# Email: valinvescap@gmail.com

##############################
# Standard library imports
##############################

from unittest import TestCase

import numpy as np

from gdelt.dateFuncs import _gdeltRangeString, _dateRanger
from gdelt.vectorizingFuncs import _urlBuilder


##############################
# Third party imports
##############################
##############################
# Local imports
##############################


class TestUrlBuilder(TestCase):
    def test_urlbuilder_v1(self):
        date_sequence = '2016 10 01'
        ranger_output = _dateRanger(date_sequence)
        gdeltstring_output = _gdeltRangeString(ranger_output, version=1)
        urlbuilder_test = _urlBuilder(gdeltstring_output, version=1)
        exp = 'http://data.gdeltproject.org/events/20161001.export.CSV.zip'
        return self.assertEqual(exp, urlbuilder_test, "Version 1 Url works.")

    def test_urlbuilder_v2(self):
        date_sequence = '2016 10 01'
        ranger_output = _dateRanger(date_sequence)
        gdeltstring_output = _gdeltRangeString(ranger_output, version=2)
        urlbuilder_test = _urlBuilder(gdeltstring_output, version=2)
        exp = 'http://data.gdeltproject.org/gdeltv2/20161001234500.export.CSV.zip'

        return self.assertEqual(exp, urlbuilder_test, "Version 2 Url works.")

    def test_urlbuilder_v2_translation(self):
        date_sequence = '2016 10 01'
        ranger_output = _dateRanger(date_sequence)
        gdeltstring_output = _gdeltRangeString(ranger_output, version=2)
        urlbuilder_test = _urlBuilder(gdeltstring_output, version=2, translation=True)
        exp = 'http://data.gdeltproject.org/gdeltv2/20161001234500.translation.export.CSV.zip'

        return self.assertEqual(exp, urlbuilder_test, "Version 2 Url works.")

    def test_urlbuilder_v3(self):
        date_sequence = '2013 Mar 01'
        ranger_output = _dateRanger(date_sequence)
        gdeltstring_output = _gdeltRangeString(ranger_output, version=1)
        urlbuilder_test = _urlBuilder(gdeltstring_output, version=1)
        exp = 'http://data.gdeltproject.org/events/201303.zip'
        return self.assertEqual(exp, urlbuilder_test, "Version 1 Url works.")

    def test_urlbuilder_v4(self):
        date_sequence = '2001 Mar 01'
        ranger_output = _dateRanger(date_sequence)
        gdeltstring_output = _gdeltRangeString(ranger_output, version=1)
        urlbuilder_test = _urlBuilder(gdeltstring_output, version=1)
        exp = 'http://data.gdeltproject.org/events/2001.zip'
        return self.assertEqual(exp, urlbuilder_test, "Version 1 Url works.")

    def test_urlbuilder_v5(self):
        date_sequence = '2013 Apr 01'
        ranger_output = _dateRanger(date_sequence)
        gdeltstring_output = _gdeltRangeString(ranger_output, version=1)
        urlbuilder_test = _urlBuilder(gdeltstring_output, table='gkg',version=1)
        exp = 'http://data.gdeltproject.org/gkg/20130401.gkg.csv.zip'
        return self.assertEqual(exp, urlbuilder_test, "Version 1 Url works.")

    def test_urlbuilder_graph2_pass(self):
        date_sequence = '2015 Apr 01'
        ranger_output = _dateRanger(date_sequence)
        gdeltstring_output = _gdeltRangeString(ranger_output, version=2)
        urlbuilder_test = _urlBuilder(gdeltstring_output, table='gkg', version=2)
        exp = 'http://data.gdeltproject.org/gdeltv2/20150401234500.gkg.csv.zip'
        return self.assertEqual(exp, urlbuilder_test, "Version 2 Url works.")

    def test_urlbuilder_graph2_pass_translation(self):
        date_sequence = '2015 Apr 01'
        ranger_output = _dateRanger(date_sequence)
        gdeltstring_output = _gdeltRangeString(ranger_output, version=2)
        urlbuilder_test = _urlBuilder(gdeltstring_output, table='gkg', version=2, translation=True)
        exp = 'http://data.gdeltproject.org/gdeltv2/20150401234500.translation.gkg.csv.zip'
        return self.assertEqual(exp, urlbuilder_test, "Version 2 Url works with translation.")

    def test_urlbuilder_events2_passlist(self):
        date_sequence = ['2015 Apr 01','2015 Apr 02']
        ranger_output = _dateRanger(date_sequence)
        gdeltstring_output = _gdeltRangeString(ranger_output, version=2)
        urlbuilder_test = _urlBuilder(gdeltstring_output, table='events', version=2)
        exp = np.sort(np.array(['http://data.gdeltproject.org/gdeltv2/20150401234500.export.CSV.zip',
                                'http://data.gdeltproject.org/gdeltv2/20150402234500.export.CSV.zip']))
        return np.testing.assert_array_equal(np.sort(np.array(urlbuilder_test)), np.sort(np.array(exp)))

    def test_urlbuilder_events2_passlist_translation(self):
        date_sequence = ['2015 Apr 01','2015 Apr 02']
        ranger_output = _dateRanger(date_sequence)
        gdeltstring_output = _gdeltRangeString(ranger_output, version=2)
        urlbuilder_test = _urlBuilder(gdeltstring_output, table='events', version=2, translation=True)
        exp = np.sort(np.array(['http://data.gdeltproject.org/gdeltv2/20150401234500.translation.export.CSV.zip',
                                'http://data.gdeltproject.org/gdeltv2/20150402234500.translation.export.CSV.zip']))
        return np.testing.assert_array_equal(np.sort(np.array(urlbuilder_test)), np.sort(np.array(exp)))

    def test_urlbuilder_events1_passlist(self):
        date_sequence = ['2015 Apr 01', '2015 Apr 02']
        ranger_output = _dateRanger(date_sequence)
        gdeltstring_output = _gdeltRangeString(ranger_output, version=1)
        urlbuilder_test = _urlBuilder(gdeltstring_output, table='events', version=1)
        exp = np.sort(np.array(['http://data.gdeltproject.org/events/20150401.export.CSV.zip',
                                'http://data.gdeltproject.org/events/20150402.export.CSV.zip']))
        return np.testing.assert_array_equal(np.sort(np.array(exp)), np.sort(np.array(urlbuilder_test)))



#########################
# Standard Library Import
#########################

import os
from unittest import TestCase

import pandas as pd

import gdelt
from gdelt.multipdf import _parallelize_dataframe


#########################
# Third Party Import
#########################
############################
# Custom Import
############################
class testMultiDf(TestCase):

    def test_multidf_gpd_creation(self):
        """Test creating a geometry column"""


        # loading the mock dataframe return
        events2 = pd.read_pickle(os.path.join(
                gdelt.base.BASE_DIR, "data",
                "events2samp.gz"),
                compression="gzip")

        try:
            # Applying the function
            events2 = events2.\
                assign(geometry=_parallelize_dataframe(events2))
            return self.assertTrue('geometry' in events2.columns)
        except:
            exp= ('You need to install shapely to use this feature.')
            with self.assertRaises(Exception) as context:
                events2 = events2. \
                    assign(geometry=_parallelize_dataframe(events2))
            the_exception = context.exception


            return self.assertIsInstance(str(the_exception),str, "Not installed")






#!/usr/bin/python
# -*- coding: utf-8 -*-

# Author:
# Linwood Creekmore
# Email: valinvescap@gmail.com

##############################
# Standard Library Import
##############################

try:
    from unittest import TestCase, mock
except:
    from unittest import TestCase
    import mock

import os
import sys
from io import BytesIO, StringIO
from unittest import TestCase

import pandas
import pandas as pd

import gdelt
from gdelt.getHeaders import _events2Heads, _events1Heads, _gkgHeads, \
    _mentionsHeads


##############################
# Third Party Libraries
##############################
##############################
# Custom Library Import
##############################


class testGdeltGetHeaders(TestCase):
    @mock.patch.object(gdelt.getHeaders.requests, 'get')
    def test_events2_headers(self, mock_B):
        """Return csv."""

        spam = pandas.read_csv(
            os.path.join(gdelt.base.BASE_DIR, 'data', 'events2.csv'),encoding='utf-8')
        spam.columns = ['tableId', 'dataType',
                        'Description']
        spam = spam.assign(Empty='NULLABLE')[['tableId', 'dataType', 'Empty', 'Description']]
        ver = sys.version_info.major
        if ver == 3:
            buffer = StringIO()
            spam.to_csv(buffer, index=False)
            data = buffer.getvalue().encode('utf-8')
        else:
            buffer = BytesIO()
            spam.to_csv(buffer, index=False,encoding='utf-8')
            data = buffer.getvalue()



        # df_bytes = spam.to_string(index=False).encode('utf-8')



        response = mock_B()
        response.content = data

        res = _events2Heads()

        return self.assertIsInstance(res, list, "Returned dataframe")

    @mock.patch.object(gdelt.getHeaders.requests, 'get')
    def test_events1_headers(self, mock_B):
        """Return csv."""

        spam = pandas.read_csv(
            os.path.join(gdelt.base.BASE_DIR, 'data', 'events1.csv'))[['name']]
        spam.columns = ['tableId']
        df_bytes = spam.to_string(index=False).encode('utf-8')

        response = mock_B()
        response.content = df_bytes

        res = _events1Heads()
        ver = sys.version_info.major

        exp = pd.read_csv(BytesIO(response.content))
        print(exp)

        return self.assertIsInstance(exp, pandas.DataFrame,
                                     "Returned dataframe")

    @mock.patch.object(gdelt.getHeaders.requests, 'get')
    def test_mentions_headers(self, mock_B):
        """Return csv."""

        spam = pandas.read_csv(
            os.path.join(gdelt.base.BASE_DIR, 'data', 'mentions.csv'))
        spam.columns = ['tableId', 'dataType',
                        'Description']
        ver = sys.version_info.major
        if ver == 3:
            buffer = StringIO()
            spam.to_csv(buffer,sep='\t', index=False)
            data = buffer.getvalue().encode('utf-8')
        else:
            buffer = BytesIO()
            spam.to_csv(buffer,sep='\t', index=False, encoding='utf-8')
            data = buffer.getvalue()


        # buffer = StringIO()
        # spam.to_csv(buffer, '\t')
        # # df_bytes = spam.to_string(index=False).encode('utf-8')
        # data = buffer.getvalue().encode('utf-8')
        response = mock_B()
        response.content = data

        res = _mentionsHeads()
        # exp = pd.read_csv(response.content)

        return self.assertIsInstance(spam, pandas.DataFrame,
                                     "Returned dataframe")

    @mock.patch.object(gdelt.getHeaders.requests, 'get')
    def test_gkg_headers(self, mock_B):
        """Return csv."""

        spam = pandas.read_csv(
            os.path.join(gdelt.base.BASE_DIR, 'data', 'gkg2.csv'))
        spam.columns = ['tableId', 'dataType',
                        'Description']
        ver = sys.version_info.major
        if ver == 3:
            buffer = StringIO()
            spam.to_csv(buffer,sep='\t', index=False)
            data = buffer.getvalue().encode('utf-8')
        else:
            buffer = BytesIO()
            spam.to_csv(buffer,sep='\t', index=False, encoding='utf-8')
            data = buffer.getvalue()

        # buffer = StringIO()
        # spam.to_csv(buffer, '\t')
        # # df_bytes = spam.to_string(index=False).encode('utf-8')
        # data = buffer.getvalue().encode('utf-8')
        response = mock_B()
        response.content = data

        res = _gkgHeads()
        # exp = pd.read_csv(response.content)

        return self.assertIsInstance(spam, pandas.DataFrame,
                                     "Returned dataframe")



# #!/usr/bin/python
# # -*- coding: utf-8 -*-
#
# # Author:
# # Linwood Creekmore
# # Email: valinvescap@gmail.com
#
# ##############################
# # Standard Library Import
# ##############################
#
# import os
# from unittest import TestCase
#
# ##############################
# # Third Party Libraries
# ##############################
#
# import pandas as pd
# import numpy as np
# import coveralls
#
# ##############################
# # Custom Library Import
# ##############################
#
# import gdelt
# from gdelt.base import codes, BASE_DIR
#
#
# class testTableValues(TestCase):
#     def test_codedataframe(self):
#         """Test CAMEO Code dataframe."""
#
#         f = os.path.join(BASE_DIR, 'data', 'cameoCodes.json')
#         resp = pd.read_json(f, dtype={'cameoCode': 'str', "GoldsteinScale": np.float64})
#         resp.set_index('cameoCode', drop=False, inplace=True)
#         print("This is {}".format(gdelt.__file__))
#         return (self.assertTrue(resp.equals(codes)))
#
#     def test_events1_columns(self):
#         """Test events 1 column descriptions"""
#
#         f = os.path.join(BASE_DIR, 'data', 'events1.csv')
#         resp = pd.read_csv(f)
#         tabs = gdelt.tableInfo(version=1)
#         ret = tabs.gettable('events')
#         return (self.assertTrue(resp.equals(ret)))
#
#     def test_events2_columns(self):
#         """Test events version 2 column descriptions"""
#
#         f = os.path.join(BASE_DIR, 'data', 'events2.csv')
#         resp = pd.read_csv(f)
#         tabs = gdelt.tableInfo(version=2)
#         ret = tabs.gettable('events')
#         return (self.assertTrue(resp.equals(ret)))
#
#     def test_mentions_columns_pass(self):
#         """Test mentions version 2 pass column descriptions"""
#
#         f = os.path.join(BASE_DIR, 'data', 'mentions.csv')
#         resp = pd.read_csv(f)
#         tabs = gdelt.tableInfo(version=2)
#         ret = tabs.gettable('mentions')
#         return (self.assertTrue(resp.equals(ret)))
#
#     def test_mentions_columns_fail(self):
#         """Fail mentions version 2 pass column descriptions"""
#
#         f = os.path.join(BASE_DIR, 'data', 'mentions.csv')
#         resp = pd.read_csv(f)
#         tabs = gdelt.tableInfo(version=1)
#         exp = 'GDELT 1.0 does not have a mentions table.'
#         with self.assertRaises(Exception) as context:
#             checked = tabs.gettable('mentions')
#         the_exception = context.exception
#         return self.assertEqual(exp, str(the_exception), "Exception for wrong table name.")
#
#     def test_gkg_columns_pass(self):
#         """Test gkg version 2 pass column descriptions"""
#
#         f = os.path.join(BASE_DIR, 'data', 'gkg2.csv')
#         resp = pd.read_csv(f)
#         tabs = gdelt.tableInfo(version=2)
#         ret = tabs.gettable('gkg')
#         return (self.assertTrue(resp.equals(ret)))
#
#     def test_vgkg_columns_pass(self):
#         """Test visual gkg version 2 pass column descriptions"""
#
#         f = os.path.join(BASE_DIR, 'data', 'visualgkg.csv')
#         resp = pd.read_csv(f)
#         tabs = gdelt.tableInfo(version=2)
#         ret = tabs.gettable('vgkg')
#         return (self.assertTrue(resp.equals(ret)))
#
#     def test_iatv_columns_pass(self):
#         """Test iatv pass column descriptions"""
#
#         f = os.path.join(BASE_DIR, 'data', 'iatv.csv')
#         resp = pd.read_csv(f)
#         tabs = gdelt.tableInfo(version=2)
#         ret = tabs.gettable('iatv')
#         return (self.assertTrue(resp.equals(ret)))
#
#



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Author:
# Linwood Creekmore
# Email: valinvescap@gmail.com

##################################
# Standard Library imports
##################################
import datetime

############################
# Third Party imports
#############################
import numpy as np
from dateutil.parser import parse
from gdelt.helpers import _testdate

################################
# Local Imports
################################
from gdelt.multipdf import _parallelize_dataframe




def _urlBuilder(dateString, version, table='events', translation=False):
    """
    Takes date string from gdeltRange string and creates GDELT urls


    Parameters
    ------------

    table types:
                * events and mentions (default)
                * gkg
                * mentions only
                :type version: object
                :param dateString:
                :param version:
                :param table:
    """

    if version == 2:
        base = "http://data.gdeltproject.org/gdeltv2/"

    if version == 1:
        base = "http://data.gdeltproject.org/"

    if table == "events":
        if version == 1:
            base += 'events/'
        if not translation:
            caboose = ".export.CSV.zip"
        else:
            caboose = ".translation.export.CSV.zip"
    elif table == "mentions":
        if not translation:
            caboose = ".mentions.CSV.zip"
        else:
            caboose = ".translation.mentions.CSV.zip"
    elif table == "gkg":
        if version == 1:
            base += 'gkg/'
            if isinstance(dateString,str):
                comp = _testdate(dateString)
                if comp < parse('2013 Apr 1'):
                    raise Exception('GDELT 1.0 Global Knowledge Graph requires dates greater'
                                    ' than or equal to April 1 2013')
            elif isinstance(dateString, list) or isinstance(dateString,
                                                          np.ndarray):
                if not (np.all(list(
                        map(
                            lambda x: x > parse('2013 04 01'), list(
                                map(
                                    _testdate, dateString)))))):
                    raise Exception('GDELT 1.0 Global Knowledge Graph requires dates greater'
                                    ' than or equal to April 1 2013')


        if not translation:
            caboose = ".gkg.csv.zip"
        else:
            caboose = ".translation.gkg.csv.zip"
    else:  # pragma: no cover
        raise ValueError('You entered an incorrect GDELT table type.'
                         ' Choose between \"events\",\"mentions\",'
                         'and \"gkg\".')


    if isinstance(dateString, list) is True or isinstance(dateString,
                                                          np.ndarray) is True:
        # print("This is before any changes {}".format(dateString))
        newdate = []
        olddateString = dateString
        date=dateString
        for l in date:
            if len(l) == 4:  # pragma: no cover
                test = (str(datetime.datetime.strptime(l, '%Y')))
                newdate.append(test)
            elif len(l) == 6:  # pragma: no cover
                test = str(datetime.datetime.strptime(l, '%Y%m'))
                newdate.append(test)
            else:

                test = str(parse(str(l)))
                newdate.append(test)

        if version ==1:
            if table != 'gkg':
                # print(base)
                base
                # print("After {}".format(base))

        if not (np.all(list(
                map(
                    lambda x: x > parse('2013 04 01'), list(
                        map(
                            _testdate, dateString)))))):  # pragma: no cover

            return (list(
                map(lambda x: base + x + ".zip" if _testdate(
                    x).date() < parse(
                    '2013 04 01').date() else base + x + caboose,
                    dateString)))

        else:

            return list(map(lambda x: base + x + caboose, olddateString))

    elif isinstance(dateString, str) is True or len(dateString) == 1:

        if version == 1:
            if table=='events':
                if len(dateString) == 4:
                    comp = datetime.datetime.strptime(dateString,'%Y')
                elif len(dateString) == 6:
                    comp = datetime.datetime.strptime(dateString, '%Y%m')
                else:
                    comp = parse(dateString)
                if comp < parse('2013 Apr 01'):
                    caboose = ".zip"
                elif table == 'events':
                    caboose = ".export.CSV.zip"

        if isinstance(dateString, list) is True or isinstance(
                dateString, np.ndarray) is True:  # pragma: no cover
            dateString = dateString[0]
            if parse(dateString[0]) < parse('2013 Apr 01'):
                caboose = ".zip"

        return base + dateString + caboose

def _geofilter(frame):
    """Filters dataframe for conversion to geojson or shapefile"""
    try:
        import geopandas as gpd

        # Remove rows with no latitude and longitude
        try:

            filresults = frame[(frame['ActionGeo_Lat'].notnull()
                                ) | (frame['ActionGeo_Long'].notnull()
                                     )]
        except:

            filresults = frame[(frame['actiongeolat'].notnull()
                                ) | (frame['actiongeolong'].notnull()
                                     )]
        gdf = gpd.GeoDataFrame(filresults.assign(geometry=_parallelize_dataframe(filresults)),
                               crs={'init': 'epsg:4326'})
        gdf.columns = list(map(lambda x: (x.replace('_', "")).lower(), gdf.columns))

        final = gdf[gdf.geometry.notnull()]

        return final


    except BaseException as err:  # pragma: no cover
        if str(err) == "import of 'geopandas' halted; None in sys.modules":
            raise ImportError("geopandas is not installed. gdeltPyR needs"
                              " geopandas to export as shapefile. Visit http://"
                              "geopandas.org/install.html for instructions.")
        else:
            raise ValueError("geopandas is installed but the geospatial "
                             "functions failed for some other reason. Review"
                             " the stack trace to see where the failure "
                             "occurred.")



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Author:
# Linwood Creekmore
# Email: valinvescap@gmail.com

##############################
# Standard Library Imports
##############################

import datetime
import multiprocessing
import re
import time
import warnings
from io import BytesIO
from multiprocessing import freeze_support

##############################
# Third party Library Imports
##############################

import pandas as pd
import requests


# class NoDaemonProcess(multiprocessing.Process):
#     # make 'daemon' attribute always return False
#     def _get_daemon(self):
#         return False
#
#     def _set_daemon(self, value):
#         pass
#
#     daemon = property(_get_daemon, _set_daemon)
#
#
# # We sub-class multiprocessing.pool.Pool instead of multiprocessing.Pool
# # because the latter is only a wrapper function, not a proper class.
# class NoDaemonProcessPool(multiprocessing.pool.Pool):
#     Process = NoDaemonProcess


def _mp_worker(url, table=None, proxies=None):
    """Code to download the urls and blow away the buffer to keep memory usage
     down"""

    warnings.filterwarnings("ignore",
                            '.*have mixed types. Specify dtype.*')  # ignore
    # pandas warning for GDELT 1.0 dtype
    # start = datetime.datetime.now()
    # proc_name = current_process().name
    #
    # proc = os.getpid()

    #################################
    # DEBUG Prints
    #################################

    # print(url)
    # print(multiprocessing.current_process().name)
    # print('Starting {0}-{1}'.format(proc_name, proc))
    time.sleep(0.001)
    # print("Getting to request process finished in {}".\
    # format(datetime.datetime.now() - start))
    # start = datetime.datetime.now()
    #################################

    r = requests.get(url, proxies=proxies, timeout=5)
    # print("Request finished in {}".format(datetime.datetime.now() - start))
    if r.status_code == 404:
        message = "GDELT does not have a url for date time " \
                  "{0}".format(re.search('[0-9]{4,18}', url).group())
        warnings.warn(message)

    # print (multiprocessing.Process(name=multiprocessing.current_process().\
    # name).is_alive())
    start = datetime.datetime.now()
    try:
        buffer = BytesIO(r.content)
        if table == 'events':

            frame = pd.read_csv(buffer, compression='zip', sep='\t',
                                header=None, on_bad_lines='skip',
                                dtype={26: 'str', 27: 'str', 28: 'str'})  # ,
            # parse_dates=[1, 2])

        elif table == 'gkg':
            frame = pd.read_csv(buffer, compression='zip', sep='\t',
                                header=None,on_bad_lines='skip')
            # parse_dates=['DATE'], warn_bad_lines=False)

        else:  # pragma: no cover

            frame = pd.read_csv(buffer, compression='zip', sep='\t',
                                header=None, on_bad_lines='skip')

        # print("Pandas load finished in {}".\
        # format(datetime.datetime.now() - start))
        # print ("{0} with id {1} finished processing in {2}".\
        # format(proc_name,proc,end))
        buffer.flush()
        buffer.close()
        return frame

    except:
        try:
            message = "GDELT did not return data for date time " \
                      "{0}".format(re.search('[0-9]{4,18}', url).group())
            warnings.warn(message)
        except:  # pragma: no cover
            message = "No data returned for {0}".format(r.url)
            warnings.warn(message)


if __name__ == '__main__':  # pragma: no cover
    freeze_support()
    p = multiprocessing.Process(target=_mp_worker)
    p.start()
    # # Wait 10 seconds for foo
    # # time.sleep(35)
    # #
    # # # Terminate foo
    # # p.terminate()
    #
    # # Cleanup
    # p.join()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Author:
# Linwood Creekmore
# Email: valinvescap@gmail.com

#########################
# Standard Library Import
#########################
import datetime
from unittest import TestCase

import numpy as np

from gdelt.dateFuncs import _dateRanger


#########################
# Third Party Import
#########################
############################
# Custom Import
############################


class TestDateRanger(TestCase):
    def test_dateRanger_sequence(self):
        date_sequence = ['2016 10 01', '2016 10 05']
        ranger_test = _dateRanger(date_sequence)
        exp = (np.array([datetime.datetime(2016, 10, 1, 0, 0),
                         datetime.datetime(2016, 10, 2, 0, 0),
                         datetime.datetime(2016, 10, 3, 0, 0),
                         datetime.datetime(2016, 10, 4, 0, 0),
                         datetime.datetime(2016, 10, 5, 0, 0)], dtype=object))
        return np.testing.assert_array_equal(exp, ranger_test)

    def test_dateRanger_single_list(self):
        date_sequence = ['2016 10 01']
        ranger_test = _dateRanger(date_sequence)
        exp = (np.array(datetime.datetime(2016, 10, 1, 0, 0), dtype=object))
        return np.testing.assert_array_equal(exp, ranger_test)

    def test_dateRanger_single_string(self):
        date_string = '2016 10 01'
        ranger_test = _dateRanger(date_string)
        exp = (np.array(datetime.datetime(2016, 10, 1, 0, 0), dtype=object))
        return np.testing.assert_array_equal(exp, ranger_test)

    def test_separate_dates(self):
        """Tests more than 3 dates"""

        date_sequence = ['2016 Jun 10','2016 Jun 15','2016 Jun 25']
        ranger_test = _dateRanger(date_sequence)
        exp = (np.array([datetime.datetime(2016, 6, 10, 0, 0),
               datetime.datetime(2016, 6, 15, 0, 0),
               datetime.datetime(2016, 6, 25, 0, 0)], dtype=object))
        return np.testing.assert_array_equal(exp, ranger_test)









#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import absolute_import, print_function

import codecs
import os
import re

from setuptools import setup

cwd = os.path.abspath(os.path.dirname(__file__))


def read(filename):
    with codecs.open(os.path.join(cwd, filename), 'rb', 'utf-8') as h:
        return h.read()


metadata = read(os.path.join(cwd, 'gdelt', '__init__.py'))


def extract_metaitem(meta):
    # swiped from https://hynek.me 's attr package
    meta_match = re.search(
        r"""^__{meta}__\s+=\s+['\"]([^'\"]*)['\"]""".format(meta=meta),
        metadata, re.MULTILINE)
    if meta_match:
        return meta_match.group(1)
    raise RuntimeError('Unable to find __{meta}__ string.'.format(meta=meta))


setup(
    name='gdelt',
    version=extract_metaitem('version'),
    license=extract_metaitem('license'),
    description=extract_metaitem('description'),
    long_description=(read('README.rst') + '\n\n' +
                      read('AUTHORS.rst') + '\n\n' +
                      read('CHANGES')),
    author=extract_metaitem('author'),
    author_email=extract_metaitem('email'),
    maintainer=extract_metaitem('author'),
    maintainer_email=extract_metaitem('email'),
    url=extract_metaitem('url'),
    # download_url=extract_metaitem('download_url'),
    platforms=['Any'],
    packages=['gdelt'],
    install_requires=['numpy>=1.11.1', 'pandas>=0.23', 'requests',
                      'python-dateutil',
                      'mock;python_version<"3.3"',
                      "futures; python_version < '3.5'",
                      "futures>=3.0.5; python_version == '2.6'"
                      " or python_version=='2.7'"
                      ],
    extras_require={
        'geoSpatial': ["fiona>=1.6", "shapely>=1.5", "geopandas>1.7"],
        ":python_version<'3.3'": ["mock", "futures"],
        'r': ['pyarrow']
    },
    include_package_data=True,
    package_data={'utils': ['schema_csvs/*']},
    data_files=[('data', ['data/cameoCodes.json', 'data/events1.csv',
                          'data/events2.csv', 'data/gkg2.csv',
                          'data/iatv.csv', 'data/mentions.csv',
                          'data/visualgkg.csv', 'data/gkg2listsamp.gz',
                          'data/events2listsamp.gz', 'data/events1.csv',
                          'data/events1samp.gz',
                          'data/events2.csv',
                          'data/events2listsamp.gz',
                          'data/events2samp.gz',
                          'data/events2samps.csv.zip',
                          'data/events2Transsamp.gz',
                          'data/gkg1.csv',
                          'data/gkg1samp.gz',
                          'data/gkg2.csv',
                          'data/gkg2listsamp.gz',
                          'data/gkg2samp.gz',
                          'data/gkg2Transsamp.gz',
                          'data/iatv.csv',
                          'data/mentions.csv',
                          'data/mentionslistsamp.gz',
                          'data/mentionssamp.csv',
                          'data/mentionssamp.gz',
                          'data/mentionsTranssamp.gz',
                          'data/visualgkg.csv'
                          ])],
    setup_requires=['pytest-runner'],
    tests_require=['pytest'],
keywords='gdelt pandas tidy data api',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Science/Research',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Operating System :: OS Independent',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: Communications :: Chat',
        'Topic :: Internet',
        'Topic :: Scientific/Engineering :: Information Analysis',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6'
    ],
)



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Author:
# Linwood Creekmore
# Email: valinvescap@gmail.com

##############################
# Standard Library Import
##############################

import os
import json
from unittest import TestCase


##############################
# Third Party Import
##############################

import numpy as np
import pandas as pd

##############################
# Custom Library Import
##############################
#
import gdelt
from gdelt.base import codes, BASE_DIR


##############################
# Third Party Libraries
##############################

class TestBaseValues(TestCase):
    def test_codedataframe(self):
        """Test install data"""

        f = os.path.join(BASE_DIR, 'data', 'cameoCodes.json')
        resp = pd.read_json(f,dtype={'cameoCode': 'str', "GoldsteinScale": np.float64})
        resp.set_index('cameoCode', drop=False, inplace=True)
        print(resp.columns,resp.index,resp.shape)
        code_test = pd.DataFrame(codes).astype(dtype={'cameoCode': 'str', "GoldsteinScale": np.float64})
        code_test.set_index('cameoCode', drop=False, inplace=True)
        code_test = code_test.sort_index()
        print(code_test.columns,code_test.index,code_test.shape)
        return (self.assertTrue(resp.equals(code_test)))

    def test_proxy_dict(self):
        "Test if proxy is dictionary"

        proxies = {"http": None, "https": None}
        gd = gdelt.gdelt(proxies=proxies)
        self.assertTrue(isinstance(gd.proxies, dict), "Test for dictonary not "
                                                      "working.")

    def test_proxy_dict_fail(self):
        exp = ("The proxies parameter must be a dictionary. See http://docs."
               "python-requests.org/en/master/user/advanced/#proxies for "
               "more information.")
        with self.assertRaises(Exception) as context:
            gd = gdelt.gdelt(proxies="nope")
        the_exception = context.exception
        return self.assertEqual(exp, str(the_exception), "The proxy dictionary"
                                                         " error failed.")



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Author:
# Linwood Creekmore
# Email: valinvescap@gmail.com

##################################
# Standard library imports
##################################

try:
    from unittest import TestCase, mock
except:
    from unittest import TestCase
    import mock

import os
from unittest import TestCase

import pandas as pd

import gdelt


##############################
# Third party imports
##############################

import geopandas as gpd


##############################
# Local imports
##############################


class TestGdeltBaseSearch(TestCase):
    @mock.patch('gdelt.gdelt.Search')
    def test_search_events_fail_nodata(self, mock_get):
        """Send a request to the API server and store the response."""

        # gd1 = gdelt.gdelt(version=2)
        #
        # # loading the mock dataframe return
        # mockin = mock.Mock(side_effect=ValueError("This GDELT query returned no data. Check internet connection or query parameters and retry"))
        # # Creating the mock object
        #
        #
        # # Adding attributes for the mock; setting it to the pickled dataframe
        # mock_get.return_value.dataframe = exp
        #
        # # real response for testing
        # response = gd1.Search('2017 July', table='events', coverage=False)
        #
        # # Use dataframe method to test for equality
        # return (self.assertTrue(response.dataframe.equals(events1)))

    @mock.patch.object(gdelt.gdelt, 'Search')
    def test_search_events_fail_nodata(self, response):
        """Raise exception with empty dataframe."""

        gd = gdelt.gdelt(version=2)
        gd.Search('2017 27 Jul').return_value = pd.DataFrame()
        gd.Search('2017 27 Jul').side_effect = ValueError("This GDELT query "
                                                          "returned no "
                                                          "data. Check query "
                                                          "parameters "
                                                          "and retry")

        exp = "This GDELT query returned no data. Check query parameters and" \
              " retry"


        with self.assertRaises(Exception) as context:
            gd.Search('2017 Jul 27')()

        the_exception = context.exception

        return self.assertEqual(exp, str(the_exception), "Raise exception for"
                                                         " zero data return.")

    # @mock.patch.object(gdelt.gdelt, 'Search')
    # def test_search_table_fail_name(self, response):
    #     """Fail on using an invalid table name"""
    #
    #     table = 'even'
    #     valid = ['events', 'gkg', 'vgkg', 'iatv', 'mentions']
    #
    #     if table not in valid:
    #         gd = gdelt.gdelt(version=2)
    #         gd.Search('2017 27 Jul', table='even'). \
    #             side_effect = ValueError('"You entered "event"; this is not '
    #                                      'a valid table name. Choose from '
    #                                      '"events", "mentions", or "gkg".')
    #
    #     exp = ('"You entered "event"; this is not a valid table name. '
    #            'Choose from "events", "mentions", or "gkg".')
    #     with self.assertRaises(Exception) as context:
    #         gd.Search('2017 Jul 27', table='event')()
    #     the_exception = context.exception
    #     return self.assertEqual(exp, str(the_exception),
    #                             "Exception for wrong table name.")

    def test_basedir(self):
        """Fail on using an invalid table name"""

        exp = (gdelt.base.BASE_DIR)
        this_dir, this_filename = os.path.split(__file__)
        BASE_DIR = os.path.dirname(this_dir)
        BASE = "/".join(this_dir.split('/')[:-1])
        return self.assertIsInstance(BASE_DIR,str)

    def test_Search_load(self):
        """Just load the class"""

        gd = gdelt.gdelt()
        return self.assertIsInstance(gd,gdelt.gdelt)

    # patching in order; bottom to top is first to last
    @mock.patch.object(gdelt.base,'_mp_worker')
    @mock.patch.object(gdelt.base,'_events2Heads')
    def test_my_method_mock_events2_pass(self, mock_B,mock_A):

        # instantiating the first mock; need the headers
        mock_B = mock.MagicMock(return_value=pd.read_csv(
            (os.path.join(gdelt.base.BASE_DIR, 'data', 'events2.csv')))[
            'name'].values.tolist())

        # instantiating the second mock
        mock_A = mock.MagicMock(return_value=(
            pd.read_pickle(os.path.join(
                gdelt.base.BASE_DIR, "data", "events2samp.gz"),
                compression="gzip")))

        gd = gdelt.gdelt(version=2)

        res = gd.Search('2017 Jul 1', table='events')

        # returning the objects
        return self.assertGreater(mock_A.return_value.shape[0], 0)

        # patching in order; bottom to top is first to last
    @mock.patch.object(gdelt.base, '_mp_worker')
    @mock.patch.object(gdelt.base, '_events2Heads')
    def test_my_method_mock_events2_fail_table_name(self, mock_B, mock_A):
        # instantiating the first mock; need the headers
        mock_B = mock.MagicMock(return_value=pd.read_csv(
            (os.path.join(gdelt.base.BASE_DIR, 'data', 'events2.csv')))[
            'name'].values.tolist())

        # instantiating the second mock
        mock_A = mock.MagicMock(return_value=(
            pd.read_pickle(os.path.join(
                gdelt.base.BASE_DIR, "data", "events2samp.gz"),
                compression="gzip")))

        gd = gdelt.gdelt(version=2)
        with self.assertRaises(Exception) as context:
            res = gd.Search('2017 Jul 1', table='evens')

        the_exception = context.exception
        exp = (
        """You entered "evens"; this is not a valid table name. Choose from "events", "mentions", or "gkg".""")


        # returning the objects
        return self.assertEqual(exp,str(context.exception))


    # patching in order; bottom to top is first to last
    @mock.patch.object(gdelt.base, '_mp_worker')
    @mock.patch.object(gdelt.base, '_gkgHeads')
    def test_my_method_mock_gkgv2_pass(self, mock_B, mock_A):
        # instantiating the first mock; need the headers
        mock_B = mock.MagicMock(return_value=pd.read_csv(
            (os.path.join(gdelt.base.BASE_DIR, 'data', 'gkg2.csv')))[
            'name'].values.tolist())

        # instantiating the second mock
        mock_A = mock.MagicMock(return_value=(
            pd.read_pickle(os.path.join(
                gdelt.base.BASE_DIR, "data" ,"gkg2samp.gz"),
                compression="gzip")))

        # print(mock_A.return_value.columns)
        gd=gdelt.gdelt(version=2)

        res = gd.Search('2017 Jul 1',table='gkg')

        # returning the objects
        return self.assertGreater(mock_A.return_value.shape[0], 0)

        # patching in order; bottom to top is first to last
    @mock.patch.object(gdelt.base, '_mp_worker')
    @mock.patch.object(gdelt.base, '_mentionsHeads')
    def test_my_method_mock_events1_pass(self, mock_B, mock_A):
        # instantiating the first mock; need the headers
        mock_B = mock.MagicMock(return_value=pd.read_csv(
            (os.path.join(gdelt.base.BASE_DIR, 'data', 'events1.csv')))[
            'name'].values.tolist())

        # instantiating the second mock
        mock_A = mock.MagicMock(return_value=(
            pd.read_pickle(os.path.join(
                gdelt.base.BASE_DIR, "data", "events1samp.gz"),
                compression="gzip")))

        # print(mock_A.return_value.columns)
        gd = gdelt.gdelt(version=1)

        res = gd.Search('2017 Jul 1', table='events')

        # returning the objects
        return self.assertGreater(mock_A.return_value.shape[0], 0)

    @mock.patch.object(gdelt.base, '_mp_worker')
    @mock.patch.object(gdelt.base, '_mentionsHeads')
    def test_my_method_mock_events1_fail_mentions(self, mock_B, mock_A):
        # instantiating the first mock; need the headers
        mock_B = mock.MagicMock(return_value=pd.read_csv(
            (os.path.join(gdelt.base.BASE_DIR, 'data', 'events1.csv')))[
            'name'].values.tolist())

        # instantiating the second mock
        mock_A = mock.MagicMock(return_value=(
            pd.read_pickle(os.path.join(
                gdelt.base.BASE_DIR, "data", "events1samp.gz"),
                compression="gzip")))

        # print(mock_A.return_value.columns)
        gd = gdelt.gdelt(version=1)

        with self.assertRaises(Exception) as context:
            res = gd.Search('2017 Jul 1', table='mentions')

        the_exception = context.exception
        exp = ('GDELT 1.0 does not have the "mentions"'
                                    ' table. Specify the "events" or "gkg"'
                                    'table.')


        # returning the objects
        return self.assertEqual(exp,str(context.exception))

    @mock.patch.object(gdelt.base, '_mp_worker')
    @mock.patch.object(gdelt.base, '_mentionsHeads')
    def test_my_method_mock_events1_fail_translation(self, mock_B, mock_A):
        # instantiating the first mock; need the headers
        mock_B = mock.MagicMock(return_value=pd.read_csv(
            (os.path.join(gdelt.base.BASE_DIR, 'data', 'events1.csv')))[
            'name'].values.tolist())

        # instantiating the second mock
        mock_A = mock.MagicMock(return_value=(
            pd.read_pickle(os.path.join(
                gdelt.base.BASE_DIR, "data", "events1samp.gz"),
                compression="gzip")))

        # print(mock_A.return_value.columns)
        gd = gdelt.gdelt(version=1)

        with self.assertRaises(Exception) as context:
            res = gd.Search('2017 Jul 1', translation=True)

        the_exception = context.exception
        exp = ('GDELT 1.0 does not have an option to'
                                    ' return translated table data. Switch to '
                                    'version 2 by reinstantiating the gdelt '
                                    'object with <gd = gdelt.gdelt(version=2)>')

        # returning the objects
        return self.assertEqual(exp, str(context.exception))

    @mock.patch.object(gdelt.base, '_mp_worker')
    @mock.patch.object(gdelt.base, '_events1Heads')
    def test_my_method_mock_mentions_pass(self, mock_B, mock_A):
        # instantiating the first mock; need the headers
        mock_B = mock.MagicMock(return_value=pd.read_csv(
            (os.path.join(gdelt.base.BASE_DIR, 'data', 'mentions.csv')))[
            'name'].values.tolist())

        # instantiating the second mock
        mock_A = mock.MagicMock(return_value=(
            pd.read_pickle(os.path.join(
                gdelt.base.BASE_DIR, "data", "mentionssamp.gz"),
                compression="gzip")))

        # print(mock_A.return_value.columns)
        gd = gdelt.gdelt(version=2)

        res = gd.Search('2017 Jul 1', table='mentions')

        # returning the objects
        return self.assertGreater(mock_A.return_value.shape[0], 0)

    @mock.patch.object(gdelt.base, '_mp_worker')
    def test_my_method_mock_gkgv1_pass(self, mock_B):
        # instantiating the first mock; need the headers
        # mock_B = mock.MagicMock(return_value=pd.read_csv(
        #     (os.path.join(gdelt.base.BASE_DIR, 'data', 'mentions.csv')))[
        #     'name'].values.tolist())

        # instantiating the second mock
        mock_A = mock.MagicMock(return_value=(
            pd.read_pickle(os.path.join(
                gdelt.base.BASE_DIR, "data", "gkg1samp.gz"),
                compression="gzip")))

        # print(mock_A.return_value.columns)
        gd = gdelt.gdelt(version=1)

        res = gd.Search('2017 Jul 1', table='gkg')

        # returning the objects
        return self.assertGreater(mock_A.return_value.shape[0], 0)

    @mock.patch.object(gdelt.base, '_mp_worker')
    @mock.patch.object(gdelt.base, '_events1Heads')
    def test_my_method_mock_eventsv2_trans_pass(self, mock_B, mock_A):
        # instantiating the first mock; need the headers
        mock_B = mock.MagicMock(return_value=pd.read_csv(
            (os.path.join(gdelt.base.BASE_DIR, 'data', 'events2.csv')))[
            'name'].values.tolist())

        # instantiating the second mock
        mock_A = mock.MagicMock(return_value=(
            pd.read_pickle(os.path.join(
                gdelt.base.BASE_DIR, "data", "events2Transsamp.gz"),
                compression="gzip")))

        # print(mock_A.return_value.columns)
        gd = gdelt.gdelt(version=2)

        res = gd.Search('2017 Jul 1', table='events',translation=True)

        # returning the objects
        return self.assertGreater(mock_A.return_value.shape[0], 0)

    @mock.patch.object(gdelt.base, '_mp_worker')
    @mock.patch.object(gdelt.base, '_mentionsHeads')
    def test_my_method_mock_mentions_trans_pass(self, mock_B, mock_A):
        # instantiating the first mock; need the headers
        mock_B = mock.MagicMock(return_value=pd.read_csv(
            (os.path.join(gdelt.base.BASE_DIR, 'data', 'mentions.csv')))[
            'name'].values.tolist())

        # instantiating the second mock
        mock_A = mock.MagicMock(return_value=(
            pd.read_pickle(os.path.join(
                gdelt.base.BASE_DIR, "data",
                "mentionsTranssamp.gz"),
                compression="gzip")))

        # print(mock_A.return_value.columns)
        gd = gdelt.gdelt(version=2)

        res = gd.Search('2017 Jul 1', table='mentions', translation=True)

        # returning the objects
        return self.assertGreater(mock_A.return_value.shape[0], 0)

    @mock.patch.object(gdelt.base, '_mp_worker')
    @mock.patch.object(gdelt.base, '_gkgHeads')
    def test_my_method_mock_gkg2_trans_pass(self, mock_B, mock_A):
        # instantiating the first mock; need the headers
        mock_B = mock.MagicMock(return_value=pd.read_csv(
            (os.path.join(gdelt.base.BASE_DIR, 'data', 'gkg2.csv')))[
            'name'].values.tolist())

        # instantiating the second mock
        mock_A = mock.MagicMock(return_value=(
            pd.read_pickle(os.path.join(
                gdelt.base.BASE_DIR, "data",
                "gkg2Transsamp.gz"),
                compression="gzip")))

        # print(mock_A.return_value.columns)
        gd = gdelt.gdelt(version=2)

        res = gd.Search('2017 Jul 1', table='gkg', translation=True)

        # returning the objects
        return self.assertGreater(mock_A.return_value.shape[0], 0)

    @mock.patch.object(gdelt.base, '_mp_worker')
    @mock.patch.object(gdelt.base, '_events2Heads')
    @mock.patch.object(gdelt.base,'Pool')
    @mock.patch.object(gdelt.base.pd, 'concat')
    def test_my_method_mock_events2_list_pass(self, mock_D,mock_C,mock_B, mock_A):
        # instantiating the first mock; need the headers

        mock_C = mock.MagicMock(return_value=(
            pd.read_pickle(os.path.join(
                gdelt.base.BASE_DIR, "data",
                "events2listsamp.gz"),
                compression="gzip")))

        mock_C = mock.MagicMock(return_value=(
            pd.read_pickle(os.path.join(
                gdelt.base.BASE_DIR, "data",
                "events2listsamp.gz"),
                compression="gzip")))

        mock_B = mock.MagicMock(return_value=pd.read_csv(
            (os.path.join(gdelt.base.BASE_DIR, 'data', 'events2.csv')))[
            'name'].values.tolist())

        # instantiating the second mock

        mock_A = mock.MagicMock(return_value=(
            pd.read_pickle(os.path.join(
                gdelt.base.BASE_DIR, "data",
                "events2listsamp.gz"),
                compression="gzip")))

        # print(mock_A.return_value.columns)
        gd = gdelt.gdelt(version=2)

        res = gd.Search(date=['2017 May 1','2017 Jun 1','2017 Jul 1'])

        # returning the objects
        return self.assertGreater(mock_A.return_value.shape[0], 0)

    @mock.patch.object(gdelt.base, '_mp_worker')
    @mock.patch.object(gdelt.base, '_mentionsHeads')
    @mock.patch.object(gdelt.base, 'Pool')
    @mock.patch.object(gdelt.base.pd, 'concat')
    @mock.patch.object(gdelt.base.concurrent.futures, 'ProcessPoolExecutor')
    def test_my_method_mock_mentions_list_pass(self, mock_E, mock_D, mock_C, mock_B,
                                              mock_A):
        # instantiating the first mock; need the headers

        mock_E = mock.MagicMock(return_value=(
            pd.read_pickle(os.path.join(
                gdelt.base.BASE_DIR, "data",
                "mentionslistsamp.gz"),
                compression="gzip")))

        mock_C = mock.MagicMock(return_value=(
            pd.read_pickle(os.path.join(
                gdelt.base.BASE_DIR, "data",
                "mentionslistsamp.gz"),
                compression="gzip")))

        mock_C = mock.MagicMock(return_value=(
            pd.read_pickle(os.path.join(
                gdelt.base.BASE_DIR, "data",
                "mentionslistsamp.gz"),
                compression="gzip")))

        mock_B = mock.MagicMock(return_value=pd.read_csv(
            (os.path.join(gdelt.base.BASE_DIR, 'data', 'mentions.csv')))[
            'name'].values.tolist())

        # instantiating the second mock

        mock_A = mock.MagicMock(return_value=(
            pd.read_pickle(os.path.join(
                gdelt.base.BASE_DIR, "data",
                "mentionslistsamp.gz"),
                compression="gzip")))

        # print(mock_A.return_value.columns)
        gd = gdelt.gdelt(version=2)

        res = gd.Search(date=['2017 May 1', '2017 Jun 1', '2017 Jul 1'],table='mentions')

        # returning the objects
        return self.assertGreater(mock_A.return_value.shape[0], 0)

    @mock.patch.object(gdelt.base, '_mp_worker')
    @mock.patch.object(gdelt.base, '_gkgHeads')
    @mock.patch.object(gdelt.base, 'Pool')
    @mock.patch.object(gdelt.base.pd, 'concat')
    @mock.patch.object(gdelt.base.concurrent.futures, 'ProcessPoolExecutor')
    def test_my_method_mock_gkg2_list_pass(self, mock_E, mock_D, mock_C,
                                               mock_B,
                                               mock_A):
        # instantiating the first mock; need the headers

        mock_E = mock.MagicMock(return_value=(
            pd.read_pickle(os.path.join(
                gdelt.base.BASE_DIR, "data",
                "gkg2listsamp.gz"),
                compression="gzip")))

        mock_C = mock.MagicMock(return_value=(
            pd.read_pickle(os.path.join(
                gdelt.base.BASE_DIR, "data",
                "gkg2listsamp.gz"),
                compression="gzip")))

        mock_C = mock.MagicMock(return_value=(
            pd.read_pickle(os.path.join(
                gdelt.base.BASE_DIR, "data",
                "gkg2listsamp.gz"),
                compression="gzip")))

        mock_B = mock.MagicMock(return_value=pd.read_csv(
            (os.path.join(gdelt.base.BASE_DIR, 'data', 'gkg2.csv')))[
            'name'].values.tolist())

        # instantiating the second mock

        mock_A = mock.MagicMock(return_value=(
            pd.read_pickle(os.path.join(
                gdelt.base.BASE_DIR, "data",
                "gkg2listsamp.gz"),
                compression="gzip")))

        # print(mock_A.return_value.columns)
        gd = gdelt.gdelt(version=2)

        res = gd.Search(date=['2017 May 1', '2017 Jun 1', '2017 Jul 1'],
                        table='gkg')

        # returning the objects
        return self.assertGreater(mock_A.return_value.shape[0], 0)


    @mock.patch.object(gdelt.base, '_mp_worker')
    @mock.patch.object(gdelt.base, '_events2Heads')
    @mock.patch.object(gdelt.base,'Pool')
    @mock.patch.object(gdelt.base.pd, 'concat')
    def test_my_method_mock_events2_list_df(self, mock_D,mock_C,mock_B, mock_A):
        # instantiating the first mock; need the headers

        mock_C = mock.MagicMock(return_value=(
            pd.read_pickle(os.path.join(
                gdelt.base.BASE_DIR, "data",
                "events2listsamp.gz"),
                compression="gzip")))

        mock_C = mock.MagicMock(return_value=(
            pd.read_pickle(os.path.join(
                gdelt.base.BASE_DIR, "data",
                "events2listsamp.gz"),
                compression="gzip")))

        mock_B = mock.MagicMock(return_value=pd.read_csv(
            (os.path.join(gdelt.base.BASE_DIR, 'data', 'events2.csv')))[
            'name'].values.tolist())

        # instantiating the second mock

        mock_A = mock.MagicMock(return_value=(
            pd.read_pickle(os.path.join(
                gdelt.base.BASE_DIR, "data",
                "events2listsamp.gz"),
                compression="gzip")))

        # print(mock_A.return_value.columns)
        gd = gdelt.gdelt(version=2)

        res = gd.Search(date=['2017 May 1','2017 Jun 1','2017 Jul 1'],
                        output='df')

        # returning the objects
        return self.assertGreater(mock_A.return_value.shape[0], 0)

    @mock.patch.object(gdelt.base, '_mp_worker')
    @mock.patch.object(gdelt.base, '_events2Heads')
    @mock.patch.object(gdelt.base,'Pool')
    @mock.patch.object(gdelt.base.pd, 'concat')
    def test_my_method_mock_events2_list_json(self, mock_D,mock_C,mock_B, mock_A):
        # instantiating the first mock; need the headers

        mock_C = mock.MagicMock(return_value=(
            pd.read_pickle(os.path.join(
                gdelt.base.BASE_DIR, "data",
                "events2listsamp.gz"),
                compression="gzip")))

        mock_C = mock.MagicMock(return_value=(
            pd.read_pickle(os.path.join(
                gdelt.base.BASE_DIR, "data",
                "events2listsamp.gz"),
                compression="gzip")))

        mock_B = mock.MagicMock(return_value=pd.read_csv(
            (os.path.join(gdelt.base.BASE_DIR, 'data', 'events2.csv')))[
            'name'].values.tolist())

        # instantiating the second mock

        mock_A = mock.MagicMock(return_value=(
            pd.read_pickle(os.path.join(
                gdelt.base.BASE_DIR, "data",
                "events2listsamp.gz"),
                compression="gzip")))

        # print(mock_A.return_value.columns)
        gd = gdelt.gdelt(version=2)

        res = gd.Search(date=['2017 May 1','2017 Jun 1','2017 Jul 1'],
                        output='json')

        # returning the objects
        return self.assertGreater(mock_A.return_value.shape[0], 0)

    @mock.patch.object(gdelt.base, '_mp_worker')
    @mock.patch.object(gdelt.base, '_events2Heads')
    @mock.patch.object(gdelt.base, 'Pool')
    @mock.patch.object(gdelt.base.pd, 'concat')
    def test_my_method_mock_events2_list_csv(self, mock_D, mock_C, mock_B,
                                            mock_A):
        # instantiating the first mock; need the headers

        mock_C = mock.MagicMock(return_value=(
            pd.read_pickle(os.path.join(
                gdelt.base.BASE_DIR, "data",
                "events2listsamp.gz"),
                compression="gzip")))

        mock_C = mock.MagicMock(return_value=(
            pd.read_pickle(os.path.join(
                gdelt.base.BASE_DIR, "data",
                "events2listsamp.gz"),
                compression="gzip")))

        mock_B = mock.MagicMock(return_value=pd.read_csv(
            (os.path.join(gdelt.base.BASE_DIR, 'data', 'events2.csv')))[
            'name'].values.tolist())

        # instantiating the second mock

        mock_A = mock.MagicMock(return_value=(
            pd.read_pickle(os.path.join(
                gdelt.base.BASE_DIR, "data",
                "events2listsamp.gz"),
                compression="gzip")))

        # print(mock_A.return_value.columns)
        gd = gdelt.gdelt(version=2)

        res = gd.Search(date=['2017 May 1', '2017 Jun 1', '2017 Jul 1'],
                        output='csv')

        # returning the objects
        return self.assertGreater(mock_A.return_value.shape[0], 0)

    @mock.patch.object(gdelt.base, '_mp_worker')
    @mock.patch.object(gdelt.base, '_events2Heads')
    @mock.patch.object(gdelt.base, 'Pool')
    @mock.patch.object(gdelt.base.pd, 'concat')
    @mock.patch.object(gdelt.base,'_geofilter')
    def test_my_method_mock_gpd_normcols(self, mock_E,mock_D, mock_C, mock_B,
                                             mock_A):
        # instantiating the first mock; need the headers
        d = pd.read_pickle(os.path.join(
                gdelt.base.BASE_DIR, "data",
                "events2listsamp.gz"),
                compression="gzip")
        mock_E = mock.MagicMock(return_value=(
            d))

        mock_C = mock.MagicMock(return_value=(
            d))

        mock_B = mock.MagicMock(return_value=pd.read_csv(
            (os.path.join(gdelt.base.BASE_DIR, 'data', 'events2.csv')))[
            'name'].values.tolist())

        # instantiating the second mock

        mock_A = mock.MagicMock(return_value=(
            d))

        # print(mock_A.return_value.columns)
        gd = gdelt.gdelt(version=2)

        res = gd.Search(date=['2017 May 1', '2017 Jun 1', '2017 Jul 1'],
                        output='gpd',normcols=True)

        # returning the objects
        return self.assertGreater(mock_A.return_value.shape[0], 0)




#!/usr/bin/python
# -*- coding: utf-8 -*-

# Author:
# Linwood Creekmore
# Email: valinvescap@gmail.com

#########################
# Standard Library Import
#########################
import datetime
from unittest import TestCase

from gdelt.dateFuncs import _parse_date


############################
# Custom Import
############################


class TestParse_date(TestCase):
    def test_parse_date(self):
        date_string = '2016 10 01'
        parsed_test = _parse_date(date_string)
        exp = datetime.datetime(2016, 10, 1, 0, 0)
        self.assertEqual(parsed_test, exp)

    def test_parse_date_fail(self):
        date_string = 'random string'
        parsed_test = _parse_date(date_string)
        exp = 'You entered an incorrect date.  Check your date format.'
        self.assertEqual(parsed_test, exp)



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Author:
# Linwood Creekmore
# Email: valinvescap@gmail.com

##################################
# Standard library imports
##################################
import datetime
import warnings

##################################
# Third party imports
##################################
import numpy as np
from dateutil.parser import parse

##################################
# Local imports
##################################
# from gdelt.vectorizingFuncs import _vectorizer


def _parse_date(date_string):
    """Convert a date string to a Python datetime object.

    Parameters
    ----------
    date_string : string
        A date, written as a string

    Returns
    -------
    datetime object
        Python datetime object.

    Example
    --------
    This is a simple function to return a datetime object from a string.

    >>> _parse_date('2016 10 01')
    datetime.datetime(2016, 10, 1, 0, 0)
    """

    try:
        return np.where(isinstance(parse(date_string), datetime.datetime),
                        parse(date_string), "Error")
    except Exception as e:
        return "You entered an incorrect date.  Check your date format."


def _dateformatter(date_string):
    """Converts arbritary date string into standardized date string

    Parameters
    ----------
    date_string : string
        A date, written as a string

    Returns
    -------
    string
        Standardized date string in YYYY-MM-DD format.

    Example
    --------
    This is a simple function to convert entered date strings into a standard date string format.

    >>> _dateformatter('2016 10 01')
    '2016-10-01'
    """
    return parse(date_string).strftime("%Y-%m-%d")


def _dateRanger(originalArray):
    """Function to vectorize date formatting function.
    Creates datetime.date objects for each day in the range
    and stores in a numpy array.

    """

    minutes = list(map(str, range(00, 60, 15)))
    hours = list(map(str, range(0, 24)))
    times = []
    for l in hours:
        if int(l) < 10:
            l = "0" + l
        for k in minutes:
            if k == "0":
                k = '00'
            times.append('{0}:{1}'.format(l, k))

    if isinstance(originalArray, str):
        """Check user input to retrieve date query."""

        return np.where(len(originalArray) == 0, "crazy",
                        _parse_date(originalArray))

    elif isinstance(originalArray, list):

        if len(originalArray) == 1:
            return np.array(parse("".join(originalArray)))
        elif len(originalArray) > 2:
            #
            #             return np.array(map(parse,originalArray),dtype='datetime64[D]').tolist()
            return np.array(list(map(lambda x: parse(x), originalArray)))
        else:

            cleaner = np.vectorize(_dateformatter)
            converted = cleaner(originalArray).tolist()
            dates = np.arange(converted[0], converted[1],
                              dtype='datetime64[D]')
            dates = list(map(lambda x: datetime.datetime.combine(
                x, datetime.datetime.min.time()), dates.tolist()))
            if len(originalArray) == 2:
                adder = np.datetime64(parse(converted[1]).date())
                adder = datetime.datetime.combine(adder.tolist(),
                                                  datetime.datetime.min.time())
                return np.append(dates,
                                 adder)  # numpy range is not endpoint inclusive
            else:
                pass
            return np.array(dates)


def _gdeltRangeString(element, coverage=None, version=2.0):
    """Takes a numpy datetime and converts to string"""

    ########################
    # Numpy datetime to object
    ########################


    ########################
    #     Current day check
    ########################


    minutes = list(map(str, range(00, 60, 15)))
    hours = list(map(str, range(0, 24)))
    times = []
    for l in hours:
        if int(l) < 10:
            l = "0" + l
        for k in minutes:
            if k == "0":
                k = '00'
            times.append('{0}:{1}'.format(l, k))

    element = element.tolist()

    hour = datetime.datetime.now().hour
    multiplier = (datetime.datetime.now().minute // 15)
    multiple = 15 * multiplier
    conditioner = multiplier + 1

    # calculate nearest 15 minute interval
    if not isinstance(element, list):


        if element.date() == datetime.datetime.now().date():
            if coverage and int(version) != 1:

                converted = np.array(
                    list(map(
                        lambda x: np.datetime64(parse(str(element) + " " + x)
                                                ).tolist().strftime(
                            '%Y%m%d%H%M%S'
                        ), times[:hour * 4 + conditioner])))
            else:

                converted = (datetime.datetime.now().replace(
                    minute=multiple, second=0)-datetime.timedelta(minutes=15)).strftime('%Y%m%d%H%M%S')

        else:
            if coverage and int(version) != 1:
                converted = restOfDay = np.array(
                    list(map(
                        lambda x: np.datetime64(parse(str(element) + " " + x)
                                                ).tolist().strftime(
                            '%Y%m%d%H%M%S'
                        ), times[:])))
            else:

                converted = element.replace(minute=int(
                    multiple), second=0).strftime('%Y%m%d%H%M%S')
                if parse(converted)< datetime.datetime.now():
                    converted = element.replace(minute=45, second=0,hour=23).strftime('%Y%m%d%H%M%S')


    #################################
    # All non-current dates section
    #################################

    else:

        ####################
        # Handling list
        ####################

        if isinstance(element, list) is True:

            #             converted = map(lambda x: x.strftime('%Y%m%d%H%M%S'),element)
            converted = list(map(lambda x: (
                datetime.datetime.combine(x, datetime.time.min) +
                datetime.timedelta(
                    minutes=45, hours=23
                )
            ).strftime('%Y%m%d%H%M%S'), element))

        else:
            converted = (datetime.datetime.combine(
                element, datetime.time.min) +
                         datetime.timedelta(
                             minutes=45, hours=23
                         )
                         ).strftime('%Y%m%d%H%M%S')

        ####################
        # Return all 15 min intervals
        ####################
        if coverage and int(version) != 1:

            converted = []
            for i in element:
                converted.append(np.array(
                    list(map(
                        lambda x: np.datetime64(parse(str(i) + " " + x)
                                                ).tolist().strftime(
                            '%Y%m%d%H%M%S'
                        ), times[:]))))
            converted = np.concatenate(converted, axis=0)
            if len(converted.tolist()) >= (5 * 192):
                warnText = ("This query will download {0} files, and likely "
                            "exhaust your memory with possibly 10s of "
                            "GBs of data in this single query.Hit Ctr-C to kill "
                            "this query if you do not want to "
                            "continue.".format(len(converted.tolist())))
                warnings.warn(warnText)

    ########################
    # Version 1 Datestrings
    #########################
    if int(version) == 1:
        if isinstance(converted, list) is True:

            converted = list(
                map(lambda x: np.where((parse(x) >= parse(
                    '2013 04 01')), parse(x).strftime('%Y%m%d%H%M%S')[:8],
                                       np.where((parse(x) < parse(
                                           '2006 01 01') and (
                                                     int(version) == 1)),
                                                parse(x).strftime(
                                                    '%Y%m%d%H%M%S')[:4],
                                                parse(x).strftime(
                                                    '%Y%m%d%H%M%S')[:6]))
                    , converted))
            converted = list(map(lambda x: x.tolist(), converted))
            converted = list(set(converted))  # account for duplicates
        else:
            converted = np.where((parse(converted) >= parse('2013 04 01')),
                                 parse(converted).strftime('%Y%m%d%H%M%S')[:8],
                                 np.where((parse(converted) < parse(
                                     '2006 01 01') and (int(version) == 1)),
                                          parse(converted).strftime(
                                              '%Y%m%d%H%M%S')[:4],
                                          parse(converted).strftime(
                                              '%Y%m%d%H%M%S')[:6])).tolist()

    return converted


# def _dateMasker(dateString, version):
#     mask = (np.where((int(version == 1) and parse(dateString) >= parse(
#         '2013 04 01')) or (int(version) == 2),
#                      _vectorizer(_gdeltRangeString, _dateRanger(dateString))[:8],
#                      np.where(int(version) == 1 and parse(
#                          dateString) < parse('2006 01'),
#                               _vectorizer(_gdeltRangeString, _dateRanger(
#                                   dateString))[:4],
#                               _vectorizer(_gdeltRangeString, _dateRanger(
#                                   dateString))[:6]))).tolist()
#     return mask

